@@ -54,11 +54,12 @@ typedef struct __builder_module {
   "if NEED_FERITE_LIB\n"                                                             \
   "libferite = -L${top_builddir}/src -lferite\n"                                     \
   "endif\n\n"                                                                        \
-  "LDFLAGS          = -L${libdir} @%s_LIBS@ $(libferite)\n"                          \
+  "LDFLAGS          = $(libferite) -L${libdir} @%s_LIBS@\n"                          \
   "INCLUDES         = -I$(top_srcdir)/include -I$(prefix)/include -I. @%s_CFLAGS@\n" \
   "DEFS             = @thread_defs@\n"                                               \
-  "\nscripts_DATA     = %s\nscriptsdir       = @FE_LIBRARY_PATH@\n\n"                \
-  "EXTRA_DIST       = %s.fec $(scripts_DATA)\n"                                      \
+  "\nscripts_DATA   = %s\nscriptsdir       = @FE_XPLAT_LIBRARY_PATH@\n\n"            \
+  "\nmodxml_DATA    = %s.xml\nmodxmldir       = @FE_LIBRARY_PATH@/module-descriptions\n" \
+  "EXTRA_DIST       = $(scripts_DATA) $(modxml_DATA)\n"                              \
   "pkgdir           = @FE_NATIVE_LIBRARY_PATH@\n"                                    \
   "pkg_LTLIBRARIES  = %s.la\n"                                                       \
   "\n"                                                                               \
@@ -78,12 +79,26 @@ typedef struct __builder_module {
    "AC_SUBST(%s_CFLAGS)\n" \
    "\nmodules=\"$modules %s\"\n"
 
+#define ModuleXMLTemplateStart \
+   "<?xml version=\"1.0\" ?>\n" \
+   "<module>\n" \
+   "    <name>%s</name>\n" \
+   "    <documentation-list>\n" 
+#define ModuleXMLTemplateEnd \
+   "    </documentation-list>\n" \
+   "    <dependance-list>\n" \
+   "    </dependance-list>\n" \
+   "</module>\n"
+
+#define ModuleXMLIncludeTemplate \
+   "        <include>%s</include>\n"
+
 void print_version();
 void print_copyright();
 void print_usage();
 void show_help();
 void parse_args( int argc, char **argv );
-char *builder_generate_current_name( int to_end );
+char *builder_generate_current_name( int to_end, int want_fe_prefix  );
 char *builder_generate_current_function_name();
 void builder_process_variable( FeriteScript *script, FeriteVariable *var, char *parent );
 void builder_process_closed_function( FeriteScript *script, FeriteFunction *fnc, char *parent );
