@@ -1,10 +1,17 @@
 #!/bin/sh
 
-root=".."
+if test ! -d generated; then
+    mkdir generated;
+fi
+cd generated
+
+
+root="../.."
 rm -rf capi/
 mkdir capi/
 
-feritedoc --file $root/src/ferite.c \
+"$1""feritedoc" --file $root/src/ferite.c \
+	--extra-file $root/src/ferite_amt.c \
     --extra-file $root/src/ferite_buffer.c \
     --extra-file $root/src/ferite_class.c \
     --extra-file $root/src/ferite_compile.c \
@@ -57,5 +64,6 @@ feritedoc --file $root/src/ferite.c \
     --extra-file $root/include/ferite/fthread.h \
     --extra-file $root/include/ferite/futils.h \
     --extra-file $root/include/ferite/fvariables.h \
+    --extra-file $root/include/ferite/famt.h \
     --template html \
     --prefix capi/
