@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -31,6 +31,16 @@
 #ifndef __FERITE_H__
 #define __FERITE_H__
 
+#define FERITE_API  extern
+#ifdef VCWIN32
+# undef  FERITE_API
+# ifdef FERITELIB_EXPORTS
+#  define FERITE_API __declspec(dllexport)
+# else
+#  define FERITE_API __declspec(dllimport)
+# endif
+#endif
+
 #define FE_USE_GENERATIONAL_GC 1
 
 #include <ferite/freq.h>
@@ -40,59 +50,37 @@
 extern "C"
 {
 #endif
-
-#include <ferite/fthread.h>
-#include <ferite/fglobals.h>
-#include <ferite/fhash.h>
-#include <ferite/fdebug.h>
-#include <ferite/fstack.h>
-#include <ferite/fns.h>
-#include <ferite/fvariables.h>
-#include <ferite/fregex.h>
-#include <ferite/fops.h>
-#include <ferite/fopcode.h>
-#include <ferite/futils.h>
-#include <ferite/fstring.h>
-#include <ferite/fmem_jedi.h>
-#include <ferite/fmem.h>
-#include <ferite/fgc.h>
-#include <ferite/ffunction.h>
-#include <ferite/fmodule.h>
-#include <ferite/fcompile.h>
-#include <ferite/foop.h>
-#include <ferite/ferror.h>
-#include <ferite/farray.h>
-#include <ferite/fbuffer.h>
   
 #define FERITE_NAME       "ferite"
-#define FERITE_VERSION    "0.99.6"
-#define FERITE_MAJOR      0
-#define FERITE_MINOR      99
-#define FERITE_RELEASE    6
+#define FERITE_VERSION    "1.1.17"
+#define FERITE_MAJOR      1
+#define FERITE_MINOR      1
+#define FERITE_RELEASE    17
    
-   int           ferite_init( int argc, char **argv );
-   void          ferite_set_script_argv( int argc, char **argv );
-   int           ferite_deinit();
-   FeriteScript *ferite_new_script();
-   int           ferite_script_load( FeriteScript *script, char *filename );
-   int           ferite_script_load_from_stream( FeriteScript *script, FILE *file );
-   FeriteScript *ferite_script_compile( char *filename );
-   FeriteScript *ferite_script_compile_with_path( char *filename, char **paths );
-   int           ferite_script_execute( FeriteScript *script );
-   FeriteScript *ferite_duplicate_script( FeriteScript *script );
-   int           ferite_script_delete( FeriteScript *script );
-   void          ferite_show_help();
+FERITE_API int           ferite_init( int argc, char **argv );
+FERITE_API int           ferite_deinit();
+FERITE_API void          ferite_set_script_argv( int argc, char **argv );
+FERITE_API FeriteScript *ferite_new_script();
+FERITE_API int           ferite_script_load( FeriteScript *script, char *filename );
+FERITE_API int           ferite_script_load_from_stream( FeriteScript *script, FILE *file );
+FERITE_API int           ferite_cache_toggle( int state );
+FERITE_API FeriteScript *ferite_script_compile( char *filename );
+FERITE_API FeriteScript *ferite_script_compile_with_path( char *filename, char **paths );
+FERITE_API int           ferite_script_execute( FeriteScript *script );
+FERITE_API FeriteScript *ferite_duplicate_script( FeriteScript *script );
+FERITE_API int           ferite_script_delete( FeriteScript *script );
+FERITE_API void          ferite_show_help();
    
 #define ferite_register_class( script, ns, name ) ferite_register_inherited_class( script, ns, name, NULL )
-   FeriteClass *ferite_register_inherited_class( FeriteScript *script, FeriteNamespace *ns, char *name, char *parent );
-   int ferite_register_class_function( FeriteScript *script, FeriteClass *classptr, FeriteFunction *f, int is_static );
-   int ferite_register_class_variable( FeriteScript *script, FeriteClass *classptr, FeriteVariable *variable, int is_static );
+FERITE_API FeriteClass *ferite_register_inherited_class( FeriteScript *script, FeriteNamespace *ns, char *name, char *parent );
+FERITE_API int ferite_register_class_function( FeriteScript *script, FeriteClass *classptr, FeriteFunction *f, int is_static );
+FERITE_API int ferite_register_class_variable( FeriteScript *script, FeriteClass *classptr, char *name, FeriteVariable *variable, int is_static );
    
    /* user functions for getting parameters */
-   int ferite_get_parameters( FeriteVariable **list, int num_args, ... );
-   int ferite_get_parameter_count( FeriteVariable **list );
-   void *ferite_get_parameter( FeriteVariable **list, int num );
-   int ferite_get_required_parameter_list_size( FeriteScript *script, FeriteFunction *function );
+FERITE_API int ferite_get_parameters( FeriteVariable **list, int num_args, ... );
+FERITE_API int ferite_get_parameter_count( FeriteVariable **list );
+FERITE_API void *ferite_get_parameter( FeriteVariable **list, int num );
+FERITE_API int ferite_get_required_parameter_list_size( FeriteScript *script, FeriteFunction *function );
    
    /* macros to cover internal functions */
 #define fe_new_lng( name, val )             ferite_create_number_long_variable( script, name, val, FE_ALLOC )
@@ -115,62 +103,100 @@ extern "C"
 #define fe_create_cls_fnc( script, class, name, function, signature, static ) ferite_register_class_function( script, class, ferite_create_external_function( script, name, function, signature ), static )
    
    /* macros to make writing functions easier */
-#define REDUCEREF( obj ) obj->refcount--
-#define INCRREF( obj )   obj->refcount++
+#define FDECREF( obj ) do { (obj)->refcount--; if( (obj)->refcount < 0 ) ferite_debug_catch((obj),(obj)->refcount); } while(0)
+#define FINCREF( obj ) (obj)->refcount++
+#define FDECREFI( obj, amount ) do { (obj)->refcount -= (amount); if( (obj)->refcount < 0 ) ferite_debug_catch((obj),(obj)->refcount); } while(0)
+#define FINCREFI( obj, amount ) (obj)->refcount += (amount)
 
    /* these are for return values */
-   
 #define FE_FALSE 0
 #define FE_TRUE  !FE_FALSE
    
-#define FE_NATIVE_FUNCTION( func_name ) FeriteVariable *func_name ( FeriteScript *script, FeriteFunction *function, FeriteVariable **params )
-   
-#define FE_RETURN_VAR( var ) { \
+#define FE_NATIVE_FUNCTION( func_name ) \
+	FERITE_API FeriteVariable *func_name ( FeriteScript *script, void *__container__, FeriteObject *current_recipient, FeriteFunction *function, FeriteVariable **params )
+
+#define FE_CONTAINER_TO_OBJECT  (FeriteObject*)__container__
+#define FE_CONTAINER_TO_CLASS   (FeriteClass*)__container__
+#define FE_CONTAINER_TO_NS      (FeriteNamespace*)__container__
+    
+#define FE_RETURN_VAR( var ) do { \
    FeriteVariable *ferite_function_return = var; \
+   FUD(("Variable %p being returned from function %s\n", var, __FUNCTION__)); \
    if( !FE_VAR_IS_DISPOSABLE( ferite_function_return ) ) \
      MARK_VARIABLE_AS_DISPOSABLE( ferite_function_return ); \
-   return ferite_function_return; }
+   return ferite_function_return; } while(0)
    
 #define FE_RETURN_LONG( val ) \
-   FE_RETURN_VAR( ferite_create_number_long_variable( script,"external_function_return_int", val, FE_STATIC ) )
+   FE_RETURN_VAR( ferite_create_number_long_variable( script,"external_function_return_int_" , val, FE_STATIC ) )
    
 #define FE_RETURN_DOUBLE( val ) \
-       FE_RETURN_VAR( ferite_create_number_double_variable( script, "external_function_return_float", val, FE_STATIC ) )
+   FE_RETURN_VAR( ferite_create_number_double_variable( script, "external_function_return_float_" , val, FE_STATIC ) )
    
-#define FE_RETURN_VOID { \
-   FeriteVariable *ferite_function_return = ferite_create_void_variable( script, "external_function_return_void", FE_STATIC ); \
-   MARK_VARIABLE_AS_DISPOSABLE( ferite_function_return ); \
-   return ferite_function_return; }
+#define FE_RETURN_VOID \
+   FE_RETURN_VAR( ferite_create_void_variable( script, "external_function_return_void_" , FE_STATIC ) );
 
-#define FE_RETURN_STR( str, freeme ) { \
-   FeriteVariable *ferite_function_return= ferite_create_string_variable( script, "external_function_return_string", str, FE_STATIC ); \
-   if( freeme == FE_TRUE ) { ferite_str_destroy( str ); } \
+#define FE_RETURN_STR( str, freeme ) do { \
+   FeriteVariable *ferite_function_return= ferite_create_string_variable( script, "external_function_return_string_" , str, FE_STATIC ); \
+   if( freeme == FE_TRUE ) { ferite_str_destroy( script, str ); } \
    MARK_VARIABLE_AS_DISPOSABLE( ferite_function_return ); \
-   return ferite_function_return; }
+   return ferite_function_return; } while(0)
 
-#define FE_RETURN_TRUE    FE_RETURN_LONG(FE_TRUE)
-#define FE_RETURN_FALSE   FE_RETURN_LONG(FE_FALSE)
+#define FE_RETURN_CSTR( str, freeme ) do { \
+    char *__real__str__ = str; \
+    FeriteVariable *ferite_function_return= ferite_create_string_variable_from_ptr( script, "external_function_return_string_" , __real__str__, 0, FE_CHARSET_DEFAULT, FE_STATIC ); \
+    if( freeme == FE_TRUE ) { ffree( __real__str__ ); } \
+    MARK_VARIABLE_AS_DISPOSABLE( ferite_function_return ); \
+    return ferite_function_return; } while(0)
+
+#define FE_RETURN_TRUE    FE_RETURN_VAR( ferite_create_boolean_variable( script,"external_function_return_bool-true_" , FE_TRUE, FE_STATIC ) )
+#define FE_RETURN_FALSE   FE_RETURN_VAR( ferite_create_boolean_variable( script,"external_function_return_bool-false_" , FE_FALSE, FE_STATIC ) )
 
 #define FE_RETURN_NULL_OBJECT \
-   FE_RETURN_VAR( ferite_create_object_variable( script, "external_function_return_null_object", FE_STATIC ) )
+   FE_RETURN_VAR( ferite_create_object_variable( script, "external_function_return_null_object_" , FE_STATIC ) )
 
-#define FE_RETURN_OBJECT( val ) { \
-   FeriteVariable *ferite_function_return= ferite_create_object_variable( script, "external_function_return_object", FE_STATIC ); \
-   if( val != NULL ) { VAO(ferite_function_return) = val; val->refcount++; } \
+#define FE_RETURN_OBJECT( val ) do { \
+   FeriteVariable *ferite_function_return= ferite_create_object_variable( script, "external_function_return_object_" , FE_STATIC ); \
+   if( val != NULL ) { VAO(ferite_function_return) = val; FINCREF(val); } \
    MARK_VARIABLE_AS_DISPOSABLE( ferite_function_return ); \
-   return ferite_function_return; }
+   return ferite_function_return; } while(0)
    
-#define FE_RETURN_ARRAY( val ) { \
-   FeriteVariable *ferite_function_return= ferite_create_uarray_variable( script, "external_function_return_array", val->size, FE_STATIC ); \
+#define FE_RETURN_ARRAY( val ) do { \
+   FeriteVariable *ferite_function_return= ferite_create_uarray_variable( script, "external_function_return_array_" , val->size, FE_STATIC ); \
    ferite_uarray_destroy( script, val ); \
-   VAUA(ferite_function_return) = ferite_uarray_dup( script, val, (void *(*)(FeriteScript*,FeriteVariable*))ferite_duplicate_variable ); \
+   VAUA(ferite_function_return) = ferite_uarray_dup( script, val ); \
    MARK_VARIABLE_AS_DISPOSABLE( ferite_function_return ); \
-   return ferite_function_return; }
+   return ferite_function_return; } while(0)
    
 #define FE_CLEAN_STRING(s)   fcalloc( s, sizeof(char) )
 
-#include <ferite/fobj.h> /* As this is the native class 'Obj' we need the macros here for compilation!*/
-    
+#define FE_NoScript          NULL
+
+#include <ferite/famt.h>
+#include <ferite/fcontainer.h>
+#include <ferite/fthread.h>
+#include <ferite/fglobals.h>
+#include <ferite/fhash.h>
+#include <ferite/fdebug.h>
+#include <ferite/fstack.h>
+#include <ferite/fns.h>
+#include <ferite/fvariables.h>
+#include <ferite/fregex.h>
+#include <ferite/fops.h>
+#include <ferite/fopcode.h>
+#include <ferite/futils.h>
+#include <ferite/fstring.h>
+#include <ferite/fmem.h>
+#include <ferite/fgc.h>
+#include <ferite/ffunction.h>
+#include <ferite/fmodule.h>
+#include <ferite/fcompile.h>
+#include <ferite/foop.h>
+#include <ferite/ferror.h>
+#include <ferite/farray.h>
+#include <ferite/fbuffer.h>
+
+#include <ferite/fobj.h> /* As this is the native class 'Obj' we need the macros here for compilation!*/    
+
 #ifdef __cplusplus
 }
 #endif
