@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -32,7 +32,9 @@
 # define __FERITE_DEBUG_H__
 
 # ifdef DEBUG
-#  define _DEBUG
+#  ifndef _DEBUG
+#    define _DEBUG
+#  endif
 /* provides slower but more secure memory tracking */
 #  define FERITE_MEM_DEBUG
 #  define FUD( var ) if( ferite_show_debug == 1 ) printf var ;
@@ -41,14 +43,26 @@
     FUD(("|%s--> %s [line %d] Entering %s()\n", ferite_stroflen(' ', (ferite_call_level*2)), __FILE__, __LINE__, __FUNCTION__)); \
     ferite_call_level++;
 
+#  define FE_ENTER_NAMED_FUNCTION( NAME ) \
+    int ferite_depth = ferite_call_level; \
+    FUD(("|%s--> %s [line %d] Entering %s()\n", ferite_stroflen(' ', (ferite_call_level*2)), __FILE__, __LINE__, NAME)); \
+    ferite_call_level++;
+
 #  define FE_LEAVE_FUNCTION( blim ) { ferite_call_level--; \
     FUD(("|%s<-- %s [line %d] Leaving  %s()\n", ferite_stroflen(' ', (ferite_call_level*2)), __FILE__, __LINE__, __FUNCTION__)); \
     if( ferite_call_level != ferite_depth ) { \
         FUD(( "Depth Error: (file %s, line %d, function %s, depth=%d, expected %d)\n", __FILE__, __LINE__, __FUNCTION__, ferite_call_level, ferite_depth ));  } \
     return blim; }
 
+#  define FE_LEAVE_NAMED_FUNCTION( NAME ) { ferite_call_level--; \
+    FUD(("|%s<-- %s [line %d] Leaving  %s()\n", ferite_stroflen(' ', (ferite_call_level*2)), __FILE__, __LINE__, NAME)); \
+    if( ferite_call_level != ferite_depth ) { \
+        FUD(( "Depth Error: (file %s, line %d, function %s, depth=%d, expected %d)\n", __FILE__, __LINE__, NAME, ferite_call_level, ferite_depth )); } }
+
 # else
 #  define FUD( var )
+#  define FE_ENTER_NAMED_FUNCTION( NAME )
+#  define FE_LEAVE_NAMED_FUNCTION( NAME )
 #  define FE_ENTER_FUNCTION
 #  define FE_LEAVE_FUNCTION( blim ) return blim;
 # endif
@@ -61,5 +75,6 @@
       }
 
 extern int call_level;
+void ferite_debug_catch( void *p, int count );
 
 #endif /* __FERITE_DEBUG_H__ */
