@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -34,16 +34,22 @@
 #define FNC_IS_INTRL 1
 #define FNC_IS_EXTRL 2
 
-FeriteFunction *ferite_create_internal_function( FeriteScript *script, char *name );
-FeriteFunction *ferite_create_external_function( FeriteScript *script, char *name, void *f, char *description );
-FeriteFunction *ferite_function_get( FeriteScript *script, char *name );
-FeriteOp **ferite_function_get_compiled_code( FeriteScript *script, char *name );
-void ferite_delete_function_list( FeriteScript *script, FeriteFunction *funcs );
-void ferite_function_to_external( FeriteScript *script, FeriteFunction *func );
-void ferite_delete_function_hash( FeriteScript *script, FeriteHash *hash );
-int ferite_check_params( FeriteScript *script, FeriteVariable **params, FeriteParameterRecord **signature );
-FeriteFunction *ferite_class_get_function(FeriteScript *script, FeriteClass *cls, char *name);
-FeriteFunction *ferite_function_dup( FeriteScript *script, FeriteFunction *function, FeriteClass *container );
-char *ferite_function_generate_sig_string( FeriteScript *script, FeriteFunction *f );
+#define FE_BY_VALUE  1
+#define FE_BY_REF    2
+
+#define FE_FUNCTION_IS_DIRECTIVE 42
+
+FERITE_API FeriteFunction *ferite_create_internal_function( FeriteScript *script, char *name );
+FERITE_API FeriteFunction *ferite_create_external_function( FeriteScript *script, char *name, void *f, char *description );
+FERITE_API FeriteFunction *ferite_create_alias_function( FeriteScript *script, FeriteFunction *to );
+FERITE_API FeriteFunction *ferite_function_get( FeriteScript *script, char *name );
+FERITE_API FeriteOp **ferite_function_get_compiled_code( FeriteScript *script, char *name );
+FERITE_API void ferite_delete_function_list( FeriteScript *script, FeriteFunction *funcs );
+FERITE_API void ferite_function_to_external( FeriteScript *script, FeriteFunction *func );
+FERITE_API void ferite_delete_function_hash( FeriteScript *script, FeriteHash *hash );
+FERITE_API int ferite_check_params( FeriteScript *script, FeriteVariable **params, FeriteFunction *function );
+FERITE_API FeriteFunction *ferite_function_dup( FeriteScript *script, FeriteFunction *function, FeriteClass *container );
+FERITE_API char *ferite_function_generate_sig_string( FeriteScript *script, FeriteFunction *f );
+FERITE_API int ferite_compare_functions( FeriteScript *script, FeriteFunction *f1, FeriteFunction *f2 );
 
 #endif /* _FERITE_FUNCTION_H_ */
