@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -31,17 +31,23 @@
 #ifndef __FERITE_GC_H__
 #define __FERITE_GC_H__
 
-void ferite_init_std_gc( FeriteScript *script );
-void ferite_deinit_std_gc( FeriteScript *script );
-void ferite_add_to_std_gc( FeriteScript *script, FeriteObject *obj );
-void ferite_check_std_gc( FeriteScript *script );
-void ferite_merge_std_gc( FeriteScript *script, void *g );
+FERITE_API void ferite_init_std_gc( FeriteScript *script );
+FERITE_API void ferite_deinit_std_gc( FeriteScript *script );
+FERITE_API void ferite_add_to_std_gc( FeriteScript *script, FeriteObject *obj );
+FERITE_API void ferite_check_std_gc( FeriteScript *script );
+FERITE_API void ferite_merge_std_gc( FeriteScript *script, void *g );
 
-void ferite_init_generation_gc( FeriteScript *script );
-void ferite_deinit_generation_gc( FeriteScript *script );
-void ferite_add_to_generation_gc( FeriteScript *script, FeriteObject *obj );
-void ferite_check_generation_gc( FeriteScript *script );
-void ferite_check_gc_generation( FeriteScript *script, FeriteGCGeneration *g );
-void ferite_merge_generation_gc( FeriteScript *script, void *g );
+FERITE_API void ferite_init_generation_gc( FeriteScript *script );
+FERITE_API void ferite_deinit_generation_gc( FeriteScript *script );
+FERITE_API void ferite_add_to_generation_gc( FeriteScript *script, FeriteObject *obj );
+FERITE_API void ferite_check_generation_gc( FeriteScript *script );
+FERITE_API void ferite_check_gc_generation( FeriteScript *script, FeriteGCGeneration *g );
+FERITE_API void ferite_merge_generation_gc( FeriteScript *script, void *g );
+
+FERITE_API void ferite_init_libgc_gc( FeriteScript *script );
+FERITE_API void ferite_deinit_libgc_gc( FeriteScript *script );
+FERITE_API void ferite_add_to_libgc_gc( FeriteScript *script, FeriteObject *obj );
+FERITE_API void ferite_check_libgc_gc( FeriteScript *script );
+FERITE_API void ferite_merge_libgc_gc( FeriteScript *script, void *g );
 
 #endif /* __FERITE_GC_H__ */
