@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -31,36 +31,44 @@
 #ifndef __FERITE_GLOBALS_H__
 #define __FERITE_GLOBALS_H__
 
-extern int ferite_call_level;
-extern int ferite_show_debug;
-extern int ferite_hide_mem_use;
-extern int ferite_use_mm_with_pcre;
-extern FeriteVariable *ferite_ARGV;
+#ifdef WIN32
+#pragma data_seg( ".GLOBALS" )
+#endif
 
-extern void  (*ferite_memory_init)(void);
-extern void  (*ferite_memory_deinit)(void);
-extern void *(*ferite_malloc)( size_t size, char *name, int line );
-extern void *(*ferite_calloc)( size_t size, size_t blk_size, char *name, int line );
-extern void *(*ferite_realloc)( void *ptr, size_t size );
-extern void  (*ferite_free)( void *ptr, char *file, int line );
+FERITE_API int ferite_call_level;
+FERITE_API int ferite_show_debug;
+FERITE_API int ferite_hide_mem_use;
+FERITE_API int ferite_use_mm_with_pcre;
+FERITE_API FeriteVariable *ferite_ARGV;
 
-extern void  (*ferite_init_gc)( FeriteScript *script );
-extern void  (*ferite_deinit_gc)( FeriteScript *script );
-extern void  (*ferite_add_to_gc)( FeriteScript *script, FeriteObject *obj );
-extern void  (*ferite_check_gc)( FeriteScript *script );
-extern void  (*ferite_merge_gc)( FeriteScript *script, void *gc );
+FERITE_API void  (*ferite_memory_init)(void);
+FERITE_API void  (*ferite_memory_deinit)(void);
+FERITE_API void *(*ferite_malloc)( size_t size, char *name, int line, FeriteScript *script );
+FERITE_API void *(*ferite_calloc)( size_t size, size_t blk_size, char *name, int line, FeriteScript *script );
+FERITE_API void *(*ferite_realloc)( void *ptr, size_t size, FeriteScript *script );
+FERITE_API void  (*ferite_free)( void *ptr, char *file, int line, FeriteScript *script );
 
+FERITE_API void  (*ferite_init_gc)( FeriteScript *script );
+FERITE_API void  (*ferite_deinit_gc)( FeriteScript *script );
+FERITE_API void  (*ferite_add_to_gc)( FeriteScript *script, FeriteObject *obj );
+FERITE_API void  (*ferite_check_gc)( FeriteScript *script );
+FERITE_API void  (*ferite_merge_gc)( FeriteScript *script, void *gc );
+
+#ifdef WIN32
+#pragma data_seg()
+#endif
 
 /* these allow you to tweak the size of hashes */
 
 #define FE_ARRAY_DEFAULT_SIZE            32
 
-#define FE_CLASS_VARIABLE_HASH_SIZE      15
-#define FE_CLASS_FUNCTION_HASH_SIZE      15
+#define FE_CLASS_VARIABLE_HASH_SIZE      36
+#define FE_CLASS_FUNCTION_HASH_SIZE      36
 
 #define FE_COMPILER_INTERNAL_STACK_SIZE  20
 
 #define FE_EXECUTOR_STACK_SIZE           32
+#define FE_DEEPEST_STACK_LEVEL           1024
 
 #define FE_FUNCTION_PARAMETER_MAX_SIZE   32
 #define FE_FUNCTION_OPCODE_INIT_SIZE     32
@@ -78,6 +86,6 @@ extern void  (*ferite_merge_gc)( FeriteScript *script, void *gc );
 
 #define FE_SEARCH_PATH_STACK_SIZE        5
 
-#define FE_CACHE_SIZE			 600
+#define FE_CACHE_SIZE                    100
 
 #endif /* __FERITE_GLOBALS_H__ */
