@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -31,20 +31,20 @@
 #ifndef __FERITE_HASH_H__
 #define __FERITE_HASH_H__
 
-unsigned int      ferite_hash_gen( char *key, int keylen );
-FeriteHashBucket *ferite_create_hash_bucket( FeriteScript *script, char *key, void *value );
-FeriteHash       *ferite_create_hash( FeriteScript *script, int size );
-FeriteHash       *ferite_hash_dup( FeriteScript *script, FeriteHash *hash, void *(*ddup)(FeriteScript *,void *,void *), void *data2 );
-void              ferite_delete_hash( FeriteScript *script, FeriteHash *hash, void (*cb)( FeriteScript *script,void *data) );
-void              ferite_process_hash( FeriteScript *script, FeriteHash *hash, void (*cb)( FeriteScript *script,void *data,char *key) );
-void              ferite_hash_add( FeriteScript *script, FeriteHash *hash, char *key, void *data );
-void              ferite_hash_delete( FeriteScript *script, FeriteHash *hash, char *key );
-void             *ferite_hash_get( FeriteScript *script, FeriteHash *hash, char *key );
-FeriteHashBucket *ferite_hash_walk(FeriteScript *script, FeriteHash *hash, FeriteIterator *iter);
-FeriteHash       *ferite_hash_grow( FeriteScript *script, FeriteHash *hash );
-void              ferite_hash_print( FeriteScript *script, FeriteHash *hash );
-void              ferite_hash_update( FeriteScript *script, FeriteHash *hash, char *key, char *data );
+FERITE_API unsigned int      ferite_hash_gen( char *key, size_t keylen );
+FERITE_API FeriteHashBucket *ferite_create_hash_bucket( FeriteScript *script, char *key, void *value );
+FERITE_API FeriteHash       *ferite_create_hash( FeriteScript *script, int size );
+FERITE_API FeriteHash       *ferite_hash_dup( FeriteScript *script, FeriteHash *hash, void *(*ddup)(FeriteScript *,void *,void *), void *data2 );
+FERITE_API void              ferite_delete_hash( FeriteScript *script, FeriteHash *hash, void (*cb)( FeriteScript *script,void *data) );
+FERITE_API void              ferite_process_hash( FeriteScript *script, FeriteHash *hash, void (*cb)( FeriteScript *script,void *data,char *key) );
+FERITE_API void              ferite_hash_add( FeriteScript *script, FeriteHash *hash, char *key, void *data );
+FERITE_API void              ferite_hash_delete( FeriteScript *script, FeriteHash *hash, char *key );
+FERITE_API void             *ferite_hash_get( FeriteScript *script, FeriteHash *hash, char *key );
+FERITE_API FeriteHashBucket *ferite_hash_walk(FeriteScript *script, FeriteHash *hash, FeriteIterator *iter);
+FERITE_API FeriteHash       *ferite_hash_grow( FeriteScript *script, FeriteHash *hash );
+FERITE_API void              ferite_hash_print( FeriteScript *script, FeriteHash *hash );
+FERITE_API void              ferite_hash_update( FeriteScript *script, FeriteHash *hash, char *key, void *data );
 
-FeriteIterator   *ferite_create_iterator( FeriteScript *script );
+FERITE_API FeriteIterator   *ferite_create_iterator( FeriteScript *script );
 
 #endif /* __FERITE_HASH_H__ */
