@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -31,17 +31,20 @@
 #ifndef __FERITE_MEMORY_H__
 #define __FERITE_MEMORY_H__
 
-#define fmalloc( size )       (ferite_malloc)( size, __FILE__, __LINE__ )
-#define fcalloc( a, b )       (ferite_calloc)( a, b, __FILE__, __LINE__ )
-#define ffree( ptr )          { (ferite_free)( ptr, __FILE__, __LINE__ ); ptr = NULL; }
-#define frealloc( ptr, size ) (ferite_realloc)( ptr, size )
+#define fmalloc( size )           (ferite_malloc)( (size), __FILE__, __LINE__, script )
+#define fmalloc_ngc( size )       (ferite_malloc)( (size), __FILE__, __LINE__, NULL )
+#define fcalloc( a, b )           (ferite_calloc)( (a), (b), __FILE__, __LINE__, script )
+#define fcalloc_ngc( a, b )       (ferite_calloc)( (a), (b), __FILE__, __LINE__, NULL )
+#define frealloc( ptr, size )     (ferite_realloc)( (ptr), (size), script )
+#define frealloc_ngc( ptr, size ) (ferite_realloc)( (ptr), (size), NULL )
+#define ffree( ptr )              { (ferite_free)( (ptr), __FILE__, __LINE__, script ); ptr = NULL; }
+#define ffree_ngc( ptr )          { (ferite_free)( (ptr), __FILE__, __LINE__, NULL ); ptr = NULL; }
 
-void  ferite_classic_memory_init(void);
-void  ferite_classic_memory_deinit(void);
-void *ferite_classic_malloc( size_t size, char *name, int line );
-void *ferite_classic_calloc( size_t size, size_t blk_size, char *name, int line );
-void *ferite_classic_realloc( void *ptr, size_t size );
-void  ferite_classic_free( void *ptr, char *file, int line );
-void ferite_classic_memory_dump();  
+FERITE_API void  ferite_classic_memory_init(void);
+FERITE_API void  ferite_classic_memory_deinit(void);
+FERITE_API void *ferite_classic_malloc( size_t size, char *name, int line, FeriteScript *script );
+FERITE_API void *ferite_classic_calloc( size_t size, size_t blk_size, char *name, int line, FeriteScript *script );
+FERITE_API void *ferite_classic_realloc( void *ptr, size_t size, FeriteScript *script );
+FERITE_API void  ferite_classic_free( void *ptr, char *file, int line, FeriteScript *script );
 
 #endif /* __FERITE_MEMORY_H__ */
