@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -31,12 +31,12 @@
 #ifndef __FERITE_MEM_JEDI_H__
 #define __FERITE_MEM_JEDI_H__
 
-void ferite_jedi_memory_init();
-void ferite_jedi_memory_deinit();
-void *ferite_jedi_malloc( size_t size, char *file, int line );
-void *ferite_jedi_calloc( size_t size, size_t blk_size, char *file, int line );
-void *ferite_jedi_realloc( void *ptr, size_t size );
-void ferite_jedi_free( void *ptr, char *file, int line );
-void ferite_jedi_morecore( int bucket );
+FERITE_API void ferite_jedi_memory_init(void);
+FERITE_API void ferite_jedi_memory_deinit(void);
+FERITE_API void *ferite_jedi_malloc( size_t size, char *file, int line, FeriteScript *script );
+FERITE_API void *ferite_jedi_calloc( size_t size, size_t blk_size, char *file, int line, FeriteScript *script );
+FERITE_API void *ferite_jedi_realloc( void *ptr, size_t size, FeriteScript *script );
+FERITE_API void ferite_jedi_free( void *ptr, char *file, int line, FeriteScript *script );
+FERITE_API void ferite_jedi_morecore( int bucket );
 
 #endif /* __FERITE_MEM_JEDI_H__ */
