@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -31,26 +31,28 @@
 #ifndef __FERITE_MODULE_H__
 #define __FERITE_MODULE_H__
 
-FeriteModule *ferite_create_module( char *name, char *filename );
-void ferite_destroy_module_list( FeriteModule *mod );
-int ferite_init_module_list();
-void ferite_deinit_module_list();
+FERITE_API FeriteModule *ferite_create_module( char *name, char *filename );
+FERITE_API void ferite_destroy_module_list( FeriteModule *mod );
+FERITE_API int ferite_init_module_list();
+FERITE_API void ferite_deinit_module_list();
+FERITE_API void ferite_clear_module_search_list();
+FERITE_API int ferite_load_module( FeriteScript *script, FeriteNamespace *ns, char *name );
+FERITE_API int ferite_load_native_module( char *name, FeriteScript *script );
+FERITE_API int ferite_load_script_module( FeriteScript *script, char *name, int do_extension );
+FERITE_API FeriteModule *ferite_module_find( char *name );
+FERITE_API int ferite_unload_native_module( char *module_name, FeriteScript *script );
 
-int ferite_load_module( FeriteScript *script, FeriteNamespace *ns, char *name );
-int ferite_load_native_module( char *name, FeriteScript *script );
-int ferite_load_script_module( FeriteScript *script, char *name, int do_extension );
-FeriteModule *ferite_module_find( char *name );
-int ferite_unload_native_module( char *module_name, FeriteScript *script );
+FERITE_API void ferite_add_library_search_path( char *path );
+FERITE_API void ferite_pop_library_search_path();
+FERITE_API void ferite_set_library_native_path( char *path );
+FERITE_API void ferite_pop_library_native_search_path();
+FERITE_API void ferite_module_register_native_function( char *lookupname, FeriteVariable *(*ptr)( FeriteScript*,void*,FeriteObject*,FeriteFunction*,FeriteVariable ** ) );
+FERITE_API void ferite_delete_native_function_record( FeriteScript *script, void *data );
+FERITE_API void ferite_module_delete_native_function_list();
+FERITE_API void *ferite_module_find_function( char *name );
 
-void ferite_add_library_search_path( char *path );
-void ferite_pop_library_search_path();
-void ferite_set_library_native_path( char *path );
-void ferite_module_register_native_function( char *lookupname, FeriteVariable *(*ptr)( FeriteScript*,FeriteFunction*,FeriteVariable ** ) );
-void ferite_delete_native_function_record( FeriteScript *script, void *data );
-void ferite_module_delete_native_function_list();
-void *ferite_module_find_function( char *name );
-
-void ferite_module_add_preload( char *name );
-int ferite_module_do_preload( FeriteScript *script );
+FERITE_API void ferite_module_add_preload( char *name );
+FERITE_API int ferite_module_do_preload( FeriteScript *script );
+FERITE_API void ferite_module_register_fake_module( char *name, void *_register, void *_unregister, void *_init, void *_deinit );
 
 #endif /* __FERITE_MODULE_H__ */
