@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -31,9 +31,21 @@
 #ifndef __FERITE_OBJ_H__
 #define __FERITE_OBJ_H__
 
-void             ferite_add_object_class( FeriteScript *script );
+void ferite_add_object_class( FeriteScript *script );
+void ferite_add_closure_class( FeriteScript *script );
 
+FE_NATIVE_FUNCTION( ferite_obj_constructor );
 FE_NATIVE_FUNCTION( ferite_obj_copy );
 FE_NATIVE_FUNCTION( ferite_obj_hash );
+FE_NATIVE_FUNCTION( ferite_obj_isSubclassOfClass );
+FE_NATIVE_FUNCTION( ferite_obj_conformsToProtocol );
+FE_NATIVE_FUNCTION( ferite_obj_getClass );
+FE_NATIVE_FUNCTION( ferite_obj_respondsToMethod );
+FE_NATIVE_FUNCTION( ferite_obj_alias );
+FE_NATIVE_FUNCTION( ferite_obj_rename );
+FE_NATIVE_FUNCTION( ferite_obj_delete );
+
+FERITE_API FeriteVariable *ferite_new_object( FeriteScript *script, FeriteClass *nclass, FeriteVariable **plist );
+FERITE_API FeriteVariable *ferite_build_object( FeriteScript *script, FeriteClass *nclass);
 
 #endif /* __FERITE_OBJ_H__ */
