@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -31,32 +31,39 @@
 #ifndef __FERITE_OPCODE_H__
 # define __FERITE_OPCODE_H__
 
-/* the instructions we have
- * Note: all instructions opdata_* is a FeriteVariable *except* CALL_FUNC */
-
-# define F_OP_NOP           0     /* no op */
-# define F_OP_BINARY        1     /* call binary opcode->op with opcode->opdata_one & two as data items */
-# define F_OP_UNARY         2     /* call unary operator opcode->op with op->opdata_one */
-# define F_OP_FUNCTION      3     /* call a function (char *)opcode->opdata_one as the
+/* the instructions we have */
+ 
+# define F_OP_NOP             0     /* no op */
+# define F_OP_BINARY          1     /* call binary opcode->op with opcode->opdata_one & two as data items */
+# define F_OP_UNARY           2     /* call unary operator opcode->op with op->opdata_one */
+# define F_OP_FUNCTION        3     /* call a function (char *)opcode->opdata_one as the
                                    * object, (char *)opcode->opdata_two as the function
                                    * job of called function to pull items off stack
                                    * and then push its result onto the stack */
-# define F_OP_METHOD        4
-# define F_OP_NEWOBJ        5
-# define F_OP_JMP           6     /* just jump */
-# define F_OP_EXIT          7     /* used as a return type thing =P ie to signal end of execution */
-# define F_OP_PUSH          8
-# define F_OP_PUSHVAR       9     /* push variable onto stack */
-# define F_OP_PUSHINDEX     10    /* push a function variable onto the stack */
-# define F_OP_PUSHATTR      11    /* push an object attribute onto the stack */
-# define F_OP_POP           12    /* pop variable from stack -> ie to get rid of shit pushed onto the stack */
-# define F_OP_BIE           13
-# define F_OP_BNE           14    /* jump if the staus flag marks the old false, else ignore and continue */
-# define F_OP_RGX           15
-# define F_OP_ERR           16
-# define F_OP_MANY          17
-# define F_OP_CASE          18
-# define F_OP_ARGS          19
+# define F_OP_METHOD          4
+# define F_OP_NEWOBJ          5
+# define F_OP_JMP             6     /* just jump */
+# define F_OP_EXIT            7     /* used as a return type thing =P ie to signal end of execution */
+# define F_OP_PUSH            8
+# define F_OP_PUSHVAR         9     /* push variable onto stack */
+# define F_OP_PUSHINDEX       10    /* push a function variable onto the stack */
+# define F_OP_PUSHATTR        11    /* push an object attribute onto the stack */
+# define F_OP_POP             12    /* pop variable from stack -> ie to get rid of shit pushed onto the stack */
+# define F_OP_BIE             13
+# define F_OP_BNE             14    /* jump if the staus flag marks the old false, else ignore and continue */
+# define F_OP_CLSRE_ASSGN     15
+# define F_OP_ERR             16
+# define F_OP_MANY            17
+# define F_OP_CASE            18
+# define F_OP_ARGS            19
+# define F_OP_DELIVER         20
+# define F_OP_SET_DELIVER     21
+# define F_OP_GET_DELIVER     22
+# define F_OP_SWAP_TOP        23
+# define F_OP_VRST            24
+# define F_OP_RAISE           25
+# define F_OP_PUSHGLOBAL      26
+# define F_OP_NOPTOP          27
 
 # define OP_MANY( p, o, d) \
    p->OP_TYPE = F_OP_MANY; \
