@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -31,14 +31,11 @@
 #ifndef __FERITE_OPS__
 #define __FERITE_OPS__
 
-#define FERITE_MANY_OP( opname ) FeriteVariable *ferite_op_##opname (FeriteScript *script, FeriteVariable **vars, int count )
-#define FERITE_BINARY_OP( opname ) FeriteVariable *ferite_op_##opname (FeriteScript *script, FeriteVariable *a, FeriteVariable *b)
-#define FERITE_UNARY_OP( opname ) FeriteVariable *ferite_op_##opname (FeriteScript *script, FeriteVariable *a)
+#define FERITE_MANY_OP( opname ) FERITE_API FeriteVariable *ferite_op_##opname (FeriteScript *script, FeriteOp *current_op, FeriteVariable **vars, int count )
+#define FERITE_BINARY_OP( opname ) FERITE_API FeriteVariable *ferite_op_##opname (FeriteScript *script, FeriteOp *current_op, FeriteVariable *a, FeriteVariable *b)
+#define FERITE_UNARY_OP( opname ) FERITE_API FeriteVariable *ferite_op_##opname (FeriteScript *script, FeriteOp *current_op, FeriteVariable *a)
 #define FERITE_OP_CALL( opname ) (void *)ferite_op_##opname
 
-FeriteVariable *ferite_new_object( FeriteScript *script, FeriteClass *nclass, FeriteVariable **plist );
-FeriteVariable *ferite_build_object( FeriteScript *script, FeriteClass *nclass);
-
 FERITE_UNARY_OP( not_op );
 FERITE_UNARY_OP( left_incr );
 FERITE_UNARY_OP( right_incr );
@@ -76,6 +73,7 @@ FERITE_BINARY_OP( greater_than_equals );
 FERITE_BINARY_OP( array_index );
 FERITE_MANY_OP( array_slice );
 FERITE_UNARY_OP( array_clear );
+FERITE_MANY_OP( array_index_assign );
 
 FERITE_BINARY_OP( left_shift );
 FERITE_BINARY_OP( right_shift );
@@ -88,8 +86,6 @@ FERITE_BINARY_OP( binary_xor_assign );
 FERITE_BINARY_OP( isa );
 FERITE_BINARY_OP( instanceof );
 
-FERITE_MANY_OP( foreach );
-
 #define FERITE_OPCODE_not_op                0
 #define FERITE_OPCODE_left_incr             1
 #define FERITE_OPCODE_right_incr            2
@@ -133,6 +129,6 @@ FERITE_MANY_OP( foreach );
 #define FERITE_OPCODE_isa                  40
 #define FERITE_OPCODE_instanceof           41
 #define FERITE_OPCODE_array_clear          42
-#define FERITE_OPCODE_foreach              43
+#define FERITE_OPCODE_array_index_assign   43
 
 #endif /* __FERITE_OPS__ */
