@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -40,19 +40,15 @@
 #define F_RGX_GLOBAL     0x0002
 #define F_RGX_EVAL_SWP   0x0004
 
-void           *ferite_regex_malloc( size_t size);
-void            ferite_regex_free( void *ptr );
-void            ferite_init_regex();
-void            ferite_deinit_regex();
-FeriteRegex    *ferite_create_regex();
-FeriteRegex *ferite_regex_dup( FeriteRegex *rgx );
-FeriteRegex    *ferite_generate_regex( FeriteScript *script, char *pattern, char *swapbuffer, char type, char *flags );
-void            ferite_delete_regex( FeriteRegex *rgx );
-char           *ferite_regex_swap_vars( char *rgxBuf, FeriteScript *script, FeriteExecuteRec *er );
-void           *ferite_compile_regex( FeriteScript *script, char *pattern, int options );
-FeriteVariable *ferite_execute_regex( FeriteRegex *rgx, FeriteVariable *target, FeriteScript *script, FeriteExecuteRec *er );
-FeriteVariable *ferite_execute_match_regex( FeriteRegex *rgx, FeriteVariable *target, int global, int should_assign, FeriteNamespace *ns, 
-											FeriteScript *script, FeriteExecuteRec *exec );
-FeriteVariable *ferite_execute_swap_regex( FeriteRegex *rgx, FeriteVariable *target, int global, FeriteNamespace *ns, FeriteScript *script );
+FERITE_API void           *ferite_regex_malloc( size_t size);
+FERITE_API void            ferite_regex_free( void *ptr );
+FERITE_API void            ferite_init_regex();
+FERITE_API void            ferite_deinit_regex();
+FERITE_API FeriteRegex    *ferite_create_regex();
+FERITE_API FeriteRegex    *ferite_regex_dup( FeriteRegex *rgx );
+FERITE_API FeriteRegex    *ferite_generate_regex( FeriteScript *script, char *pattern, char *flags );
+FERITE_API void            ferite_delete_regex( FeriteRegex *rgx );
+FERITE_API char           *ferite_regex_swap_vars( char *rgxBuf, FeriteScript *script, FeriteExecuteRec *er );
+FERITE_API void           *ferite_compile_regex( FeriteScript *script, char *pattern, int options );
 
 #endif /* __FERITE_REGEX_H__ */
