@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -31,12 +31,12 @@
 #ifndef __FERITE_STACK_H__
 #define __FERITE_STACK_H__
 
-FeriteStack *ferite_create_stack( FeriteScript *script, int size );
-void         ferite_stack_push( FeriteStack *stck, void *ptr );
-void        *ferite_stack_pop( FeriteStack *stck );
-void         ferite_delete_stack(  FeriteScript *script, FeriteStack *stack );
-void        *ferite_stack_top( FeriteStack *stck );
-void **ferite_duplicate_stack_contents( FeriteScript *script, FeriteStack *stack, void *(*ddup)( FeriteScript*,void *data,void *data2 ), void *data2 );
-FeriteStack *ferite_duplicate_stack( FeriteScript *script, FeriteStack *stack, void *(*ddup)( FeriteScript*,void *data,void *data2 ), void *data2 );
+FERITE_API FeriteStack *ferite_create_stack( FeriteScript *script, int size );
+FERITE_API void         ferite_stack_push( FeriteScript *script, FeriteStack *stck, void *ptr );
+FERITE_API void        *ferite_stack_pop( FeriteScript *script, FeriteStack *stck );
+FERITE_API void         ferite_delete_stack(  FeriteScript *script, FeriteStack *stack );
+FERITE_API void        *ferite_stack_top( FeriteScript *script, FeriteStack *stck );
+FERITE_API void **ferite_duplicate_stack_contents( FeriteScript *script, FeriteStack *stack, void *(*ddup)( FeriteScript*,void *data,void *data2 ), void *data2 );
+FERITE_API FeriteStack *ferite_duplicate_stack( FeriteScript *script, FeriteStack *stack, void *(*ddup)( FeriteScript*,void *data,void *data2 ), void *data2 );
   
 #endif /* __FERITE_STACK_H__ */
