@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -43,7 +43,9 @@ typedef struct _ferite_unified_array               FeriteUnifiedArray;
 typedef struct _ferite_function                    FeriteFunction;
 typedef struct _ferite_class                       FeriteClass;
 typedef struct _ferite_script                      FeriteScript;
+typedef struct _ferite_script_attached_data        FeriteScriptAttachedData;
 typedef struct _ferite_object                      FeriteObject;
+typedef struct _ferite_object_variable             FeriteObjectVariable;
 typedef struct _ferite_function_native_information FeriteFunctionNative;
 typedef struct _ferite_parameter_record            FeriteParameterRecord;
 typedef struct _ferite_variable                    FeriteVariable;
@@ -67,15 +69,50 @@ typedef struct _ferite_op_table                    FeriteOpTable;
 typedef struct _ferite_compile_record              FeriteCompileRecord;
 typedef struct _ferite_bk_req                      FeriteBkRequest;
 typedef struct _ferite_variable_accessors          FeriteVariableAccessors;
-typedef FeriteHash                                 FeriteVariableHash;
+typedef struct _ferite_variable_clamp              FeriteVariableClamp;
+typedef struct _ferite_variable_clamp_impl         FeriteVariableClampImplementation;
+
 typedef void (*FeriteVariableGetAccessor)(FeriteScript*,FeriteVariable*);
 typedef void (*FeriteVariableSetAccessor)(FeriteScript*,FeriteVariable*,FeriteVariable*);
-typedef void (*FeriteVariableCleanupAccessor)(FeriteScript*,void*odata);
+typedef void (*FeriteVariableCleanupAccessor)(FeriteScript*,void*);
+typedef void (*FeriteAttachedDataCleanup)(FeriteScript*,int,char*,void*);
 
-  /*
+typedef struct _ferite_amt_node                    FeriteAMTNode;
+typedef struct _ferite_amt_tree                    FeriteAMTTree;
+typedef struct _ferite_amt                         FeriteAMT;
+
+/*
  * The actual structures
  */
 
+struct _ferite_amt_node {
+	char type;
+	union {
+		FeriteAMTTree *tree;
+		struct {
+			unsigned long id;
+			void *data;
+			char *key;
+		} value;
+	} u;
+};
+struct _ferite_amt_tree {
+	unsigned int    map;
+	FeriteAMTNode **base;
+	FeriteAMTTree  *parent;
+	char            index_type;
+	char            base_size;
+};
+struct _ferite_amt {
+	FeriteAMTTree *root;
+	unsigned int (*index_function)(unsigned int, unsigned int);
+	unsigned int lower_bound;
+	unsigned int upper_bound;
+	unsigned int zero_bound;
+	unsigned int last_index;
+	unsigned int total;
+};
+
 struct _ferite_stack
 {
     int    stack_ptr; /* The point where the top of the stack points too */
@@ -85,10 +122,10 @@ struct _ferite_stack
 
 struct _ferite_string
 {
-    int   length;     /* How long the string is */
-    int   encoding;   /* What encoding the string has, eg. Latin-1 or UTF-8 etc */
-    int   pos;        /* Current position within the string */
-    char *data;       /* The strings actual data */
+    size_t  length;     /* How long the string is */
+    int     encoding;   /* What encoding the string has, eg. Latin-1 or UTF-8 etc */
+    size_t  pos;        /* Current position within the string */
+	char   *data;       /* The strings actual data */
 };
 
 struct _ferite_variable_accessors
@@ -100,22 +137,34 @@ struct _ferite_variable_accessors
     int                            owner;
 };
 
-enum
+struct _ferite_variable_clamp_impl
 {
-      FE_FLAG_DISPOSABLE = 1,   /* if we can nuke it */
-      FE_FLAG_FINAL = 2,        /* if it's a final variable */
-      FE_FLAG_FINALSET = 4,     /* if it's a final variable and has a value */
-      FE_FLAG_COMPILED = 8,     /* if it's a compiled in constant */
-      FE_FLAG_IS_STATIC = 16,   /* if the varaible is a static variable */
-      FE_FLAG_PLACEHOLDER = 32, /* if this var is just a placeholder in the stack */
-      FE_FLAG_STATIC_NAME = 64  /* if the variable name is allocated or not */
+	
+};
+
+struct _ferite_variable_clamp 
+{
+	void *cache;
+	FeriteVariableClampImplementation *implementation;
+};
+
+enum 
+{
+	FE_FLAG_DISPOSABLE = 1,   /* if we can nuke it */
+	FE_FLAG_FINAL = 2,        /* if it's a final variable */
+	FE_FLAG_FINALSET = 4,     /* if it's a final variable and has a value */
+	FE_FLAG_COMPILED = 8,     /* if it's a compiled in constant */
+	FE_FLAG_IS_STATIC = 16,   /* if the varaible is a static variable */
+	FE_FLAG_PLACEHOLDER = 32, /* if this var is just a placeholder in the stack */
+	FE_FLAG_STATIC_NAME = 64  /* if the variable name is allocated or not */
 };
 
 struct _ferite_variable
 {
     short                    type;       /* The variables type */
     short                    flags;      /* Enum flags as above */
-    char                    *name;       /* It's name - used for hashing the variable and debug */
+    short                    state;      /* Wether or not the variable is public, protected, private */
+    char                    *vname;      /* It's name - used for hashing the variable and debug */
     union
     {
         long                 lval; /* The value of the variable if it is a number [long] */
@@ -124,18 +173,20 @@ struct _ferite_variable
         FeriteObject        *oval; /* If it is an object, == NULL if it's not instantiated */
         FeriteUnifiedArray  *aval; /* The array */
         void                *pval; /* Used to sneak non-variable around the engine */
+        FeriteClass         *cval; /* A class */
+        FeriteNamespace     *nval; /* A namespace */
     }
     data;
     long                     index;     /* It's index if it is in an array */
     void                    *lock;      /* If the variable is 'atomic' this will be an AphexMutex */
+    short                    refcount;  /* The number of references to this variable */
     FeriteVariableAccessors *accessors; /* Native variable accessors */
+	FeriteVariableClamp     *clamp;
 };
 
 struct _ferite_op_function_data
 {
-    int             argument_count;  /* Number of arguements within the call */
-    int             is_autoload;     /* If the function is autoloaded */
-    FeriteFunction *function;        /* The function to call */
+    char            argument_count;  /* Number of arguements within the call */
 };
 
 struct _ferite_op
@@ -146,6 +197,7 @@ struct _ferite_op
     long  addr;                 /* The address to jump to [depending on instruction] */
     int   line;                 /* The line the op was generated from in the source - for error reporting */
     int   block_depth;          /* block depth */
+	short           flags;
 };
 
 struct _ferite_opcode_list
@@ -153,7 +205,6 @@ struct _ferite_opcode_list
     long       size;            /* Size of the opcodelist */
     long       current_op_loc;  /* The current opcode we are pointing to, used when populating the list */
     char      *filename;        /* The file in which the opcode list was generated */
-/*    FeriteOp  *current_op;*/      /* Pointer to the current op */
     FeriteOp **list;            /* The list */
 };
 
@@ -184,6 +235,7 @@ struct _ferite_hash_bucket
 struct _ferite_hash
 {
     int size;                /* Size of the hash */
+	int count;
     FeriteHashBucket **hash; /* The top level hash list */
 };
 
@@ -196,7 +248,10 @@ struct _ferite_iterator /* Used to iterate through a hash */
 struct _ferite_parameter_record /* Used in function signatures */
 {
     FeriteVariable *variable;          /* A variable do describe it */
-    int             has_default_value; /* If it has a default value - NB: not currently used */
+	char           *name;
+    char            has_default_value; /* If it has a default value - NB: not currently used */
+    char            pass_type;         /* either FE_BY_VALUE or FE_BY_REFERENCE */
+    char            is_dots;
 };
 
 struct _ferite_function_native_information /* Stores the native code info for builder and ferite */
@@ -208,89 +263,109 @@ struct _ferite_function_native_information /* Stores the native code info for bu
 
 struct _ferite_function  /* Encapsulate a native and script function */
 {
-    char                   *name;      /* Name of the function */
-    int                     type;      /* It's type - see include/ferite/ffunction.h */
-    FeriteVariable         *(*fncPtr)( FeriteScript *, FeriteFunction *, FeriteVariable ** );
+    char                    *name;      /* Name of the function */
+    char                    type;      /* It's type - see include/ferite/ffunction.h */
+    FeriteVariable         *(*fncPtr)( FeriteScript*,void*,FeriteObject*,FeriteFunction*,FeriteVariable**);
                                        /* If it is native, it's function pointer */
     FeriteFunctionNative   *native_information; /* The native infomation about the function */
     void                   *odata;     /* If we happen to have any native data */
     int                     arg_count; /* The number of arguments in the signature */
-    int                     is_static; /* If the function is a static class method */
+    char                    is_static; /* If the function is a static class method */
     FeriteParameterRecord **signature; /* Description of the functions' arguments */
     FeriteStack            *localvars; /* Local variables in the function - script only */
-    FeriteOpcodeList       *ccode;     /* The compiled function body */
+    FeriteOpcodeList       *bytecode;     /* The compiled function body */
     void                   *lock;      /* If the function is atomic this will be a AphexMutex */
     FeriteClass            *klass;     /* The class that owns this function, NULL= a namespace function */
+    char                    state;      /* The state of the function - public, protected, private */
+    char                    is_alias;   /* The structure is an alias and therefore can't have it's members freed */
     FeriteFunction         *next;
+	int                     length; /* The length of the function in the code */
+	int                     cached; /* True if the function is cached */
+	int                     return_type; 
 };
 
 struct _ferite_class
 {
-    char               *name;       /* The name of the class */
-    long                id;         /* It's unique id in the engine - runtime generated */
-    FeriteClass        *parent;     /* The class it inherits from */
-    FeriteVariableHash *variables;  /* The class's variables */
-    FeriteHash         *functions;  /* The class's functions */
-    FeriteClass        *next;       /* The next class in the list - NB: not used at the moment */
-    FeriteNamespace    *container;  /* The namespace that the class resides in */
+    char               *name;           /* The name of the class */
+    long                id;             /* It's unique id in the engine - runtime generated */
+    short               state;          /* Abstract or final or protocol */
+    void               *odata;          /* If we happen to have any native data */
+    FeriteClass        *parent;         /* The class it inherits from */
+    FeriteHash         *object_vars;    /* The object's variables */
+    FeriteHash         *class_vars;     /* The class's variables */
+    FeriteHash         *object_methods; /* The class's functions */
+    FeriteHash         *class_methods;  /* The class's functions */
+    FeriteClass        *next;           /* The next class in the list - NB: not used at the moment */
+    FeriteNamespace    *container;      /* The namespace that the class resides in */
+    FeriteStack        *impl_list;      /* List that carries the protocol list */
+	int                 cached; /* True if the function is cached */
+};
+
+struct _ferite_object_variable
+{
+    FeriteClass          *klass;
+    FeriteHash           *variables;
+    FeriteObjectVariable *parent;
 };
 
 struct _ferite_object /* an actual instance of a FeriteClass */
 {
-    char               *name;       /* The objects name -> same as the class's name */
-    int                 oid;        /* It's id -> same as the class's id */
-    void               *odata;      /* A pointer in each object that can be used by a programmer writing
-                                     * native code, this is _not_ touched by ferite. It is up to the
-                                     * programmer using it to clear things up
-                                     */
-    int                 refcount;   /* How many times the object is referenced */
-    FeriteClass        *klass;      /* the class template used */
-    FeriteVariableHash *variables;  /* Instance variables */
-    FeriteHash         *functions;  /* A pointer to the class's function hash */
+    char                 *name;       /* The objects name -> same as the class's name */
+    int                   oid;        /* It's id -> same as the class's id */
+    void                 *odata;      /* A pointer in each object that can be used by a programmer writing
+                                       * native code, this is _not_ touched by ferite. It is up to the
+                                       * programmer using it to clear things up
+                                       */
+    int                   refcount;   /* How many times the object is referenced */
+    FeriteClass          *klass;      /* the class template used */
+    FeriteObjectVariable *variables;  /* Instance variables */
+    FeriteHash           *functions;  /* A pointer to the class's function hash */
 };
 
 struct _ferite_buffer /* Used to quickly build up data without reallocing data */
 {
    /* A buffer is a block of memory where this struct is placed first, and data are
     * appended after the struct, so the 'void *ptr' must not be free'd */
-    int size;                /* The size of the this buffer page */
-    int count;               /* Number of bytes allocated from this page */
-    void *ptr;        /* Pointer to where data begins */
-    FeriteBuffer *next;      /* The next buffer in the list */
-    FeriteBuffer *current;   /* The current buffer, is only used in the first page */
+    size_t size;                /* The size of the this buffer page */
+    size_t count;               /* Number of bytes allocated from this page */
+    void *ptr;                  /* Pointer to where data begins */
+    FeriteBuffer *next;         /* The next buffer in the list */
+    FeriteBuffer *current;      /* The current buffer, is only used in the first page */
 };
 
 struct _ferite_execute_rec  /* Used in the executor to bundle information together */
 {
-    FeriteFunction     *function;       /* The function being executed */
-    FeriteVariable    **variable_list;  /* A duplicate of the function's local variable hash */
-    FeriteStack        *stack;          /* The function's local execution stack */
-    int                 block_depth;
+	FeriteExecuteRec   *parent;
+	FeriteFunction     *function;       /* The function being executed */
+	FeriteVariable    **variable_list;  /* A duplicate of the function's local variable hash */
+	FeriteStack        *stack;          /* The function's local execution stack */
+	int                 block_depth;
+	int                 line;
 };
 
 struct _ferite_gc_generation /* Used in the generational GC */
 {
-    int size;                     /* Size of the generation */
-    int next_free;                /* The next free slot in the generation */
-    FeriteObject **contents;      /* The contents holds pointers to the objects */
-    FeriteGCGeneration *younger;  /* The generation younger than this one */
-    FeriteGCGeneration *older;    /* The generation older than this one */
+	int size;                     /* Size of the generation */
+	int next_free;                /* The next free slot in the generation */
+	FeriteObject **contents;      /* The contents holds pointers to the objects */
+	FeriteGCGeneration *younger;  /* The generation younger than this one */
+	FeriteGCGeneration *older;    /* The generation older than this one */
 };
 
 struct _ferite_std_gc /* Standard 'old skool' GC */
 {
-    FeriteObject **contents; /* The contents holds pointers to the objects */
-    int            size;     /* Size of the GC */
-    int            count;    /* Number of objects within the GC */
+	FeriteObject **contents; /* The contents holds pointers to the objects */
+	int            size;     /* Size of the GC */
 };
 
 struct _ferite_thread
 {
-    void         *ctxt;    /* actually an AphexThread* */
-    FeriteScript *script;  /* The script the thread is running in */
-    FeriteObject *obj;     /* The thread object that owns the thread */
-    int           running; /* Is it running ? */
-    int           pass_exceptions; /* Should we pass exceptions in the thread onto the parent script? */
+	void         *ctxt;    /* actually an AphexThread* */
+	FeriteScript *script;  /* The script the thread is running in */
+	FeriteObject *obj;     /* The thread object that owns the thread */
+	int           running; /* Is it running ? */
+	int           referenced; /* true until a thread destructor is called */
+	int           pass_exceptions; /* Should we pass exceptions in the thread onto the parent script? */
 };
 
 struct _ferite_thread_group /* Used to group running threads so a script wont exit until they
@@ -299,6 +374,7 @@ struct _ferite_thread_group /* Used to group running threads so a script wont ex
     void         *lock;         /* The lock */
     FeriteStack  *thread_list;  /* The list of currently running threads. A thread gets added when the
                                  * .start() method gets called */
+    FeriteScript *owner; /* The script that owns this thread_group */
 };
 
 struct _ferite_script
@@ -307,6 +383,7 @@ struct _ferite_script
     char               *filename;           /* The file the script was generated from */
     char               *scripttext;         /* The contents of the script file - only used during compilation */
     FeriteNamespace    *mainns;             /* The main namespace of the script */
+	FeriteClass        *closureClassPtr;
     
     /* only these variables are altered */
     FeriteStack        *include_list;       /* Absolute path list of all included modules */
@@ -315,6 +392,7 @@ struct _ferite_script
     void               *lock;               /* Script based lock */
     FeriteThreadGroup  *thread_group;       /* ... and it's thread group */
     FeriteScript       *parent;             /* The parent script where the thread was executed from */
+	char                is_multi_thread;    /* optimisations */
     
     /* Caching stuff */
     FeriteStack        *vars;               /* Variable cache */
@@ -323,23 +401,39 @@ struct _ferite_script
     
     /* error stuff */
     char               *current_op_file;    /* File being executed in */
-    int                 current_op_line;    /* Current line */
-    int                 error_state;        /* The error state of the script */
-    int                 keep_execution;     /* If FE_FALSE the script will stop running */
-    int                 is_executing;       /* If the script is running */
-    int                 is_being_deleted;   /* If the script is being deleted */
-    int                 return_value;       /* The scripts return value */
+    unsigned int        current_op_line;    /* Current line */
+    unsigned int        error_state;        /* The error state of the script */
+    unsigned int        keep_execution;     /* If FE_FALSE the script will stop running */
+    unsigned int        is_executing;       /* If the script is running */
+    unsigned int        is_being_deleted;   /* If the script is being deleted */
+    unsigned int        return_value;       /* The scripts return value */
+    unsigned int        stack_level;        /* How deep are we in execution depth? */
     /* regex counting */
-    int                 last_regex_count;   /* The number of captured strings last time round */
+    unsigned int        last_regex_count;   /* The number of captured strings last time round */
 
     FeriteBuffer        *error;             /* The error messages */
     FeriteBuffer        *warning;           /* The warning messages */
 
     /* ferite gc */
     void               *gc;                 /* The scripts GC */
+    int                 gc_running;         /* Is the GC running ? */
+    int                 gc_count;           /* Count ops since last GC invocation */
+    void               *gc_lock;            /* GC lock */
+	FeriteExecuteRec   *gc_stack;           /* We need this to do a GC run */
     
     /* user information */
-    void               *odata;              /* A programmer can attach data to a script if they so wish */  
+    FeriteHash         *_odata;             /* A programmer can attach data to a script if they so wish */  
+    int                 odata_id;           /* Uid for the data you get */  
+                        /* Renamed from odata to _odata to deliberately break code */
+
+	FeriteAMT          *globals;
+};
+
+struct _ferite_script_attached_data 
+{
+   int                        id;
+   void                      *data;
+   FeriteAttachedDataCleanup  cleanup;
 };
 
 struct _ferite_namespace_bucket
@@ -352,7 +446,9 @@ struct _ferite_namespace
 {
     char            *name;       /* The name of the namespace */
     int              num;        /* Size of the namespace */
-    FeriteHash      *space;      /* The hash! */
+    int             *code_fork_ref;
+    FeriteHash      *data_fork;
+    FeriteHash      *code_fork; /* The code hash! */
     FeriteNamespace *container;  /* The namespace that holds this namespace */
 };
 
@@ -372,7 +468,6 @@ struct _ferite_module /* A native module that is loaded by ferite */
 struct _ferite_regex
 {
     char       *pattern;         /* original string pattern */
-    int         type;            /* type -> match or replace */
     int         pcre_options;    /* pcre options */
     int         fergx_options;   /* regex options we have to implement */
     void       *compiled_re;     /* compiled regex */
@@ -395,7 +490,7 @@ struct _ferite_unified_array
 struct _ferite_native_function_record /* This is used to register a native function when a module
                                        * is loaded */
 {
-    FeriteVariable *(*function)( FeriteScript *, FeriteFunction*, FeriteVariable ** );
+    FeriteVariable *(*function)( FeriteScript *, void *, FeriteObject*, FeriteFunction*, FeriteVariable ** );
 };
 
 struct _ferite_op_table /* Table of op-codes */
@@ -412,6 +507,12 @@ struct _ferite_compile_record /* Used in the compiler */
     FeriteClass        *cclass;
     FeriteScript       *script;
     FeriteNamespace    *ns;
+	FeriteStack        *shadowed_local_variables;
+	FeriteStack        *local_scope_frame;
+	FeriteHash         *local_variable_hash;  
+	FeriteVariable     *last_script_return;
+	int in_closure;
+	int want_container_finish;
 };
 
 #endif /* __FERITE_STRUCTS_H__ */
