@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -33,17 +33,16 @@
 
 #define fstrdup( var ) ferite_strdup( var, __FILE__, __LINE__ )
 
-char *ferite_stroflen(char c, int l);
-char *ferite_lowercase( char *str );
-char *ferite_strip_whitespace( char *str );
-int   ferite_find_string( char *haystack, char *needle );
-char *ferite_copy_string( char *str, int from, int to );
-char *ferite_delete_string( char *str, int from, int to );
-char *ferite_insert_string( char *str, char *istr, int pos );
-char *ferite_replace_string( char *str, char *pattern, char *data );
-char *ferite_strdup( char *str, char *file, int line );
-int  *ferite_int_to_ptr( int i );
-void ferite_delete_pointer( FeriteScript *script, void *ptr );
-FeriteVariable *ferite_variable_from_array( FeriteVariable **arr, char *name );
+FERITE_API char *ferite_stroflen(char c, int l);
+FERITE_API char *ferite_lowercase( char *str );
+FERITE_API char *ferite_strip_whitespace( char *str );
+FERITE_API int   ferite_find_string( char *haystack, char *needle );
+FERITE_API char *ferite_copy_string( char *str, int from, int to );
+FERITE_API char *ferite_delete_string( char *str, int from, int to );
+FERITE_API char *ferite_insert_string( char *str, char *istr, int pos );
+FERITE_API char *ferite_replace_string( char *str, char *pattern, char *data );
+FERITE_API char *ferite_strdup( char *str, char *file, int line );
+FERITE_API int  *ferite_int_to_ptr( int i );
+FERITE_API int  ferite_strcasecmp( char *left, char *right );
 
 #endif /* __FERITE_UTILS_H__ */
