@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1999-2002 Chris Ross
+ * Copyright (C) 1999-2007 Chris Ross
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,17 +27,21 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#ifndef APHEX_AS_COMPONENT
+#include "../../../config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #ifdef WIN32
 # include <windows.h>
+# include <direct.h>
 #else
 # include <unistd.h>
 # include <dirent.h>
 #endif
 
-#include "../../../config.h"
 #include "aphex.h"
 
 int __aphex_wild_match( unsigned char *m, unsigned char *n );
@@ -60,7 +64,7 @@ char            *aphex_get_temp_directory()
 #ifndef WIN32
     return strdup( "/tmp" );
 #else
-    return NULL;
+	return NULL;
 #endif
 }
 
@@ -85,10 +89,10 @@ AphexDirectory  *aphex_directory_read( char *path )
     if( dir != INVALID_HANDLE_VALUE )
 #endif
     {
-        retval = malloc( sizeof( AphexDirectory ) );
+        retval = aphex_malloc( sizeof( AphexDirectory ) );
         retval->contents = NULL;
         retval->size = 0;
-
+        retval->pos = 0;
 #ifndef WIN32
         while( (ent = readdir( dir )) )
 #else
@@ -102,7 +106,7 @@ AphexDirectory  *aphex_directory_read( char *path )
 #endif
                 {
                     retval->size++;
-                    retval->contents = (char **)realloc( retval->contents, sizeof(char *) * retval->size );
+                    retval->contents = (char **)aphex_realloc( retval->contents, sizeof(char *) * retval->size );
 #ifndef WIN32
                     retval->contents[retval->size-1] = strdup( ent->d_name );
 #else
@@ -143,10 +147,10 @@ AphexDirectory  *aphex_directory_read_with_filter( char *path, char *filter )
     if( dir != INVALID_HANDLE_VALUE )
 #endif
     {
-        retval = malloc( sizeof( AphexDirectory ) );
+        retval = aphex_malloc( sizeof( AphexDirectory ) );
         retval->contents = NULL;
         retval->size = 0;
-
+        retval->pos = 0;
 #ifndef WIN32
         while( (ent = readdir( dir )) )
 #else
@@ -169,7 +173,7 @@ AphexDirectory  *aphex_directory_read_with_filter( char *path, char *filter )
                                            , filter ) )
                     {
                         retval->size++;
-                        retval->contents = (char **)realloc( retval->contents, sizeof(char *) * retval->size );
+                        retval->contents = (char **)aphex_realloc( retval->contents, sizeof(char *) * retval->size );
 #ifndef WIN32
                         retval->contents[retval->size-1] = strdup( ent->d_name );
 #else
@@ -213,9 +217,10 @@ AphexDirectory  *aphex_directory_read_unique_prefix( char *path )
     if( dir != INVALID_HANDLE_VALUE )
 #endif
     {
-        retval = malloc( sizeof( AphexDirectory ) );
+        retval = aphex_malloc( sizeof( AphexDirectory ) );
         retval->contents = NULL;
         retval->size = 0;
+        retval->pos = 0;
 
 #ifndef WIN32
         while( (ent = readdir( dir )) )
@@ -251,7 +256,7 @@ AphexDirectory  *aphex_directory_read_unique_prefix( char *path )
                         if( !got_name )
                         {
                             retval->size++;
-                            retval->contents = (char **)realloc( retval->contents, sizeof(char *) * retval->size );
+                            retval->contents = (char **)aphex_realloc( retval->contents, sizeof(char *) * retval->size );
                             retval->contents[retval->size-1] = strdup( filename );
                         }
                     }
@@ -281,28 +286,32 @@ void             aphex_directory_delete( AphexDirectory *dir )
     if( dir != NULL )
     {
         for( i = 0; i < dir->size; i++ )
-          free( dir->contents[i] );
-        free( dir->contents );
-        free( dir );
+          aphex_free( dir->contents[i] );
+        aphex_free( dir->contents );
+        aphex_free( dir );
     }
 }
 
 char *aphex_remove_symlinks( char *filename )
 {
-    char *buffer = calloc( PATH_MAX, sizeof(char) );
+#ifndef WIN32
+    char *buffer = aphex_calloc( PATH_MAX, sizeof(char) );
     char *tmp = realpath( filename, buffer );
 
     if( tmp == NULL )
     {
-        free( buffer );
+        aphex_free( buffer );
         return strdup( filename );
     }
+#else
+    char *buffer = strdup( filename );
+#endif
     return buffer;
 }
 
 char *aphex_relative_to_absolute( char *path )
 {
-   /* ok this is what we do:
+ /* ok this is what we do:
   * if the directory starts with the dir delim, return a copy of it
   * else
   *   get current working directory
@@ -320,22 +329,29 @@ char *aphex_relative_to_absolute( char *path )
   *   /ferite/ferite/builder/builder
   */
 
-    char *cwd = calloc( PATH_MAX, sizeof(char) );
-    char *buf = calloc( strlen( path ) + PATH_MAX + 1, sizeof(char) );
-    int i = 0, j = 0;
+    char *cwd = aphex_calloc( PATH_MAX, sizeof(char) );
+    char *buf = aphex_calloc( strlen( path ) + PATH_MAX + 1, sizeof(char) );
+    size_t i = 0, j = 0;
 
+#ifndef WIN32
     if( path[0] == DIR_DELIM )
+#else
+	if( isalpha(path[0]) && path[1] == ':' && path[2] == DIR_DELIM )
+#endif
     {
-        free( buf );
+        aphex_free( buf );
         buf = strdup( path );
     }
     else
     {
+#ifdef WIN32
+#define getcwd _getcwd
+#endif
         getcwd( cwd, PATH_MAX );
         sprintf( buf, "%s%c%s", cwd, DIR_DELIM, path );
     }
 
-    for( i = 0; i < strlen( buf ); i++ )
+    for( i = 0; i < strlen( buf ) - 1; i++ )
     {
         if( buf[i] == '.' && buf[i+1] == '.' ) /* we have double dots */
         {
@@ -348,13 +364,79 @@ char *aphex_relative_to_absolute( char *path )
             i = j;
         }
     }
-    for( i = 0; i < strlen( buf ); i++ )
+    for( i = 0; i < strlen( buf ) - 1; i++ )
     {
         if( buf[i] == '.' && buf[i+1] == DIR_DELIM )        /* remove './'s */
           memmove( buf+i-1, buf+i+1, strlen(buf+i+1)+1 );
         if( buf[i] == DIR_DELIM && buf[i+1] == DIR_DELIM )  /* remove '//'s */
-          memmove( buf+i-1, buf+i+1, strlen(buf+i+1)+1 );
+          memmove( buf+i, buf+i+1, strlen(buf+i+1)+1 );
     }
-    free( cwd );
+    aphex_free( cwd );
     return buf;
 }
+
+char            *aphex_absolute_to_relative( char *path )
+{
+    char *cwd = NULL;
+    char *p = path;
+    size_t len = strlen(path);
+    size_t i = 0;
+
+    if( path[0] != DIR_DELIM )
+    {
+      return path;
+    }
+    
+    cwd = aphex_calloc( PATH_MAX, sizeof(char) );
+    getcwd( cwd, PATH_MAX );
+    if( cwd[strlen(cwd)-1] != DIR_DELIM )
+        cwd[strlen(cwd)] = DIR_DELIM;
+    
+    for( i = 0; i < len; i++ )
+    {
+	if( path[i] != cwd[i] )
+	{
+	    aphex_free( cwd );
+	    return p;
+	}
+	else
+	  p++;
+    }
+    aphex_free( cwd );
+    return path;
+}
+
+char *aphex_directory_name( char *path ) 
+{
+	int i, len = strlen( path );
+	if( path[len - 1] == DIR_DELIM )
+		return path;
+	
+	for( i = len - 1; i >= 0; i-- ) 
+    {
+		if( path[i] == DIR_DELIM )
+		{
+			char *ret = aphex_calloc( i + 2, sizeof(char) );
+			strncpy( ret, path, i + 1 );
+			return ret;
+		}
+    }
+	return NULL;
+}
+char *aphex_file_name( char *path ) 
+{
+	int i, len = strlen( path );
+	if( path[len - 1] == DIR_DELIM )
+		return path;
+	
+	for( i = len - 1; i >= 0; i-- ) 
+    {
+		if( path[i] == DIR_DELIM )
+		{
+			char *ret = aphex_calloc( len, sizeof(char) );
+			strcpy( ret, path + i + 1 );
+			return ret;
+		}
+    }
+	return NULL;
+}
