@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1999-2002 Chris Ross
+ * Copyright (C) 1999-2007 Chris Ross
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,15 +27,25 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#ifndef APHEX_AS_COMPONENT
+# include "../../../config.h"
+#else
+# ifdef VCWIN32
+#  include "snprintf.h" /* This is so that we have somethings */
+# endif
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #ifndef WIN32
 # include <unistd.h>
+#else
+# include <io.h>
 #endif
 #include <string.h>
-#include "../../../config.h"
+
 #include "aphex.h"
 #include <fcntl.h>
 
@@ -59,7 +69,7 @@ char *aphex_file_exists_path( char *pf, ... )
     {
         return file;
     }
-    free( file );
+    aphex_free( file );
     va_end( ap );
     return NULL;
 }
@@ -78,28 +88,39 @@ int aphex_file_exists( char *pf, ... )
     {
         retval = 1;
     }
-    free( file );
+    aphex_free( file );
     va_end( ap );
     return retval;
 }
 
 char *aphex_file_to_string( char *file )
 {
-    int fd = 0;
     struct stat in;
     char *data = NULL;
-    
-    if( (fd = open( file, O_RDONLY )) != -1 )
+
+#ifdef VCWIN32
+#define open  _open
+#define read  _read
+#define close _close
+#define fileno _fileno
+#endif
+ 
+	FILE *f = fopen( file, "r" );
+
+	if( f != NULL )
     {
-        fstat( fd, &in );
-        data = calloc( in.st_size + 1, sizeof(char) );
-        if( read( fd, data, in.st_size ) != -1 )
+        fstat( fileno(f), &in );
+
+        data = aphex_malloc( in.st_size + 2 );
+		memset( data, '\0', in.st_size + 2 );
+        
+		if( fread( data, sizeof(char), in.st_size, f ) != -1 )
         {
-            close( fd );
+            fclose( f );
             return data;
         }
-        free( data );
-        close( fd );
+        fclose( f );
+		aphex_free( data );
     }
     return NULL;
 }
@@ -132,7 +153,7 @@ AphexFile *aphex_open_file( char *filename, char *mode, AphexSearchList *paths )
     handle = fopen( actual_filename, mode );
     if( handle != NULL )
     {
-        file = malloc( sizeof( AphexFile ) );
+        file = aphex_malloc( sizeof( AphexFile ) );
         file->size = filestat.st_size;
         file->handle = handle;
         file->filename = strdup( actual_filename );
@@ -146,8 +167,8 @@ void             aphex_close_file( AphexFile *file )
     if( file != NULL )
     {
         fclose( file->handle );
-        free( file->filename );
-        free( file );
+        aphex_free( file->filename );
+        aphex_free( file );
     }
 }
 
@@ -162,8 +183,8 @@ size_t           aphex_read_file( AphexFile *file, char *buffer, size_t length )
 
 AphexSearchList *aphex_create_search_list()
 {
-    AphexSearchList *list = malloc( sizeof( AphexSearchList ) );
-    list->path = calloc( sizeof( char * ) * 10, sizeof(char*) );
+    AphexSearchList *list = aphex_malloc( sizeof( AphexSearchList ) );
+    list->path = aphex_calloc( sizeof( char * ) * 10, sizeof(char*) );
     list->size = 10;
     list->next_free = 0;
     return list;
@@ -176,7 +197,7 @@ void             aphex_add_to_list( AphexSearchList *list, char *path )
         if( list->next_free >= list->size )
         {
             list->size += 5;
-            list->path = realloc( list->path, sizeof( char * ) * list->size );
+            list->path = aphex_realloc( list->path, sizeof( char * ) * list->size );
         }
         D(( "aphex debug: adding '%s' to slot %d\n", path, list->next_free ));
         list->path[list->next_free++] = strdup( path );
@@ -189,9 +210,9 @@ void             aphex_delete_search_list( AphexSearchList *list )
     if( list != NULL )
     {
         for( i = 0; i < list->next_free; i++ )
-          free( list->path[i] );
-        free( list->path );
-        free( list );
+          aphex_free( list->path[i] );
+        aphex_free( list->path );
+        aphex_free( list );
     }
 }
 
