@@ -1,14 +1,15 @@
+#ifndef APHEX_AS_COMPONENT
 #include "../../../config.h"
+#endif 
 
-#ifndef USING_DARWIN
-# include <errno.h>
-#endif
+#include <errno.h>
 
 #include "poll.h"
 #include "aphex.h"
 
 int aphex_notify_can_read_with_timeout(int fd, int wait, int ce)
 {
+#ifndef WIN32
     struct pollfd pfd;
     int i;
     pfd.fd = fd;
@@ -26,6 +27,8 @@ int aphex_notify_can_read_with_timeout(int fd, int wait, int ce)
         }
         return i;
     }
+#endif
+	return 0;
 }
 
 int aphex_notify_can_read(int fd, int ce)
