@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1999-2002 Chris Ross
+ * Copyright (C) 1999-2007 Chris Ross
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,23 +27,68 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#ifndef APHEX_AS_COMPONENT
+#include "../../../config.h"
+#endif
+
+#ifdef HAVE_LIBGC
+# define GC_THREADS
+# define GC_REDIRECT_TO_LOCAL
+# include "gc.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #ifndef WIN32
 # include <unistd.h>
 #endif
 #include <string.h>
-#include "../../../config.h"
+
 #include "aphex.h"
 
+#ifdef HAVE_LIBGC
+void *aphex_malloc( size_t size ) {
+	return GC_MALLOC(size);
+}
+void *aphex_realloc( void *old, size_t new ) {
+	return GC_REALLOC(old, new);
+}
+void *aphex_calloc( size_t size, size_t block ) {
+	void *ptr = GC_MALLOC(size * block);
+	memset(ptr, 0, (size * block));
+	return ptr;
+}
+void  aphex_free( void * ptr ) { }
+char *aphex_strdup( char *original ) {
+	return GC_STRDUP(original);
+}
+#else
+void *aphex_malloc( size_t size ) {
+	return malloc(size);
+}
+void *aphex_realloc( void *old, size_t new ) {
+	return realloc(old, new);
+}
+void *aphex_calloc( size_t size, size_t block ) {
+	return calloc(size, block);
+}
+void  aphex_free( void * ptr ) { 
+	free(ptr);
+}
+char *aphex_strdup( char *original ) {
+	return strdup(original);
+}
+#endif
+
 /***************************************************
  * THREAD
  ***************************************************/
 AphexThread *aphex_thread_create()
 {
-    AphexThread *thread = malloc(sizeof(AphexThread));
+    AphexThread *thread = aphex_malloc(sizeof(AphexThread));
     return thread;
 }
 
@@ -53,7 +98,7 @@ void aphex_thread_destroy( AphexThread *thread )
     {
         if( thread->running )
           aphex_thread_stop( thread );
-        free( thread );
+        aphex_free( thread );
     }
 }
 
@@ -63,10 +108,12 @@ int aphex_thread_start( AphexThread *thread, void *(*start_routine)(void *), voi
 
     if( thread != NULL && start_routine != NULL )
     {
-        thread->running = 1;
+#ifdef USE_PTHREAD
+		thread->running = 1;
         retval = pthread_create( &(thread->thread), NULL, start_routine, arg );
         if( detach )
           pthread_detach( thread->thread );
+#endif
         if( retval != 0 )
         {
             printf( "aphex: ERROR: Not enough system resources to create thread\n" );
@@ -80,11 +127,26 @@ void aphex_thread_stop( AphexThread *thread )
 {
     if( thread != NULL )
     {
+#ifdef USE_PTHREAD
+		if( thread->running )
+			pthread_cancel( thread->thread );
+#endif
         thread->running = 0;
-        pthread_cancel( thread->thread );
     }
 }
 
+#ifdef WIN32
+void usleep( int useconds )
+{
+    if (useconds > 500)
+      sleep ((useconds+500)/1000);
+    else if (useconds > 0)
+      sleep (1);
+    else
+      sleep (0);
+}
+#endif
+
 void aphex_thread_sleep( AphexThread *thread, int msecs )
 {
     usleep( msecs * 1000 );
@@ -94,7 +156,9 @@ void aphex_thread_join( AphexThread *thread )
 {
     if( thread != NULL )
     {
-        pthread_join( thread->thread, NULL );
+#ifdef USE_PTHREAD
+		pthread_join( thread->thread, NULL );
+#endif
         thread->running = 0;
     }
 }
@@ -103,7 +167,9 @@ void aphex_thread_exit( AphexThread *thread, void *rval )
 {
     if( thread != NULL )
     {
-        pthread_exit( rval );
+#ifdef USE_PTHREAD
+		pthread_exit( rval );
+#endif
         thread->running = 0;
     }
 }
@@ -113,11 +179,13 @@ void aphex_thread_exit( AphexThread *thread, void *rval )
  ***************************************************/
 AphexMutex *aphex_mutex_create()
 {
-    AphexMutex *mutex = malloc(sizeof(AphexMutex));
+    AphexMutex *mutex = aphex_malloc(sizeof(AphexMutex));
+#ifdef USE_PTHREAD
     pthread_mutex_init( &(mutex->mutex), NULL );
-#if defined(USING_DARWIN) || defined(USING_CYGWIN) || defined(USING_SOLARIS)
+# if defined(USING_FAKE_RECURSIVE_MUTEX)
     mutex->count = -1;
     memset( &(mutex->owner), '\0', sizeof(pthread_t) );
+# endif
 #endif
     mutex->recursive = 0;
     return mutex;
@@ -125,19 +193,24 @@ AphexMutex *aphex_mutex_create()
 
 AphexMutex *aphex_mutex_recursive_create()
 {
-    AphexMutex *mutex = malloc(sizeof(AphexMutex));
+    AphexMutex *mutex = aphex_malloc(sizeof(AphexMutex));
 
-#if defined(USING_DARWIN) || defined(USING_CYGWIN) || defined(USING_SOLARIS)
+#ifdef USE_PTHREAD
+#if defined(USING_FAKE_RECURSIVE_MUTEX)
     mutex->count = 0;
     mutex->owner = NULL;
+	mutex->is_owned = 0;
     pthread_cond_init( &mutex->cond, NULL );
     pthread_mutex_init( &mutex->mutex, NULL );
 #else
     pthread_mutexattr_init( &mutex->attr );
-    pthread_mutexattr_setkind_np( &mutex->attr, PTHREAD_MUTEX_RECURSIVE_NP );
+    pthread_mutexattr_settype( &mutex->attr, PTHREAD_MUTEX_RECURSIVE);
     pthread_mutex_init( &mutex->mutex, &mutex->attr );
+    pthread_mutexattr_destroy( &mutex->attr );
 #endif
     mutex->recursive = 1;
+#endif
+
     return mutex;
 }
 
@@ -145,16 +218,20 @@ void aphex_mutex_destroy( AphexMutex *mutex )
 {
     if( mutex != NULL )
     {
-#if defined(USING_DARWIN) || defined(USING_CYGWIN) || defined(USING_SOLARIS)
+#ifdef USE_PTHREAD
+#if defined(USING_FAKE_RECURSIVE_MUTEX)
         pthread_cond_destroy( &mutex->cond );
+#else
 #endif
         pthread_mutex_destroy( &mutex->mutex );
-        free( mutex );
+        aphex_free( mutex );
+#endif
     }
 }
 
 int aphex_mutex_lock( AphexMutex *mutex )
 {
+#ifdef USE_PTHREAD
     pthread_t self = pthread_self();
 
     if( mutex != NULL )
@@ -162,20 +239,21 @@ int aphex_mutex_lock( AphexMutex *mutex )
         if( pthread_mutex_lock( &mutex->mutex ) == -1 )
           return -1;
 
-#if defined(USING_DARWIN) || defined(USING_CYGWIN) || defined(USING_SOLARIS)
+#if defined(USING_FAKE_RECURSIVE_MUTEX)
         if( mutex->recursive == 1 )
         {
             while (1)
             {
-                if( mutex->owner == self )
+                if( pthread_equal(mutex->owner, self) )
                 {
                     mutex->count++;
                     break;
                 }
-                else if( mutex->owner == NULL )
+                else if( mutex->is_owned == 0 )
                 {
                     mutex->owner = self;
                     mutex->count = 1;
+					mutex->is_owned = 1;
                     break;
                 }
                 else
@@ -188,6 +266,7 @@ int aphex_mutex_lock( AphexMutex *mutex )
         }
 #endif
     }
+#endif
     return 0;
 }
 
@@ -195,7 +274,8 @@ int aphex_mutex_unlock( AphexMutex *mutex )
 {
     if( mutex != NULL )
     {
-#if defined(USING_DARWIN) || defined(USING_CYGWIN) || defined(USING_SOLARIS)
+#ifdef USE_PTHREAD
+#if defined(USING_FAKE_RECURSIVE_MUTEX)
         if( mutex->recursive == 1 )
         {
             if( pthread_mutex_lock (&mutex->mutex) == -1 )
@@ -204,16 +284,133 @@ int aphex_mutex_unlock( AphexMutex *mutex )
             mutex->count--;
             if( mutex->count == 0 )
             {
-                mutex->owner = NULL;
+                mutex->is_owned = 0;
                 pthread_cond_signal( &mutex->cond );
             }
         }
 #endif
         pthread_mutex_unlock( &mutex->mutex );
+#endif
     }
     return 0;
 }
 
-/*
- * have a look at pthread_cond_wait
- */
+/***************************************************
+ * EVENT
+ ***************************************************/
+AphexEvent *aphex_event_create()
+{
+    AphexEvent *event = aphex_malloc(sizeof(AphexEvent));
+#ifdef USE_PTHREAD
+    if(pthread_cond_init( &(event->cond), NULL) != 0){
+        aphex_free(event);
+	return NULL;
+    }
+    if(pthread_mutex_init( &(event->mutex), NULL ) != 0){
+        aphex_free(event);
+	return NULL;
+    }    
+#endif
+    return event;
+}
+
+void aphex_event_destroy( AphexEvent *event )
+{
+    if( event != NULL )
+    {
+#ifdef USE_PTHREAD
+         pthread_cond_destroy( &event->cond );
+         pthread_mutex_destroy( &event->mutex );
+#endif
+         aphex_free(event);
+    }
+}
+
+int aphex_event_signal( AphexEvent *event )
+{
+#ifdef USE_PTHREAD
+    pthread_cond_signal(&(event->cond));
+#endif
+    return 0;
+}
+
+int aphex_event_wait( AphexEvent *event )
+{
+#ifdef USE_PTHREAD
+    pthread_cond_wait(&(event->cond), &(event->mutex));
+#endif
+    return 0;
+}
+
+#if defined(WIN32)
+
+#include <windows.h>
+#include <time.h>
+
+#ifndef __GNUC__
+#define EPOCHFILETIME (116444736000000000i64)
+#else
+#define EPOCHFILETIME (116444736000000000LL)
+#endif
+
+struct timezone {
+	int tz_minuteswest; /* minutes W of Greenwich */
+	int tz_dsttime;     /* type of dst correction */
+};
+
+__inline int gettimeofday(struct timeval *tv, struct timezone *tz)
+{
+	FILETIME        ft;
+	LARGE_INTEGER   li;
+	__int64         t;
+	static int      tzflag;
+	
+	if (tv)
+	{
+		GetSystemTimeAsFileTime(&ft);
+		li.LowPart  = ft.dwLowDateTime;
+		li.HighPart = ft.dwHighDateTime;
+		t  = li.QuadPart;       /* In 100-nanosecond intervals */
+		t -= EPOCHFILETIME;     /* Offset to the Epoch time */
+		t /= 10;                /* In microseconds */
+		tv->tv_sec  = (long)(t / 1000000);
+		tv->tv_usec = (long)(t % 1000000);
+	}
+	
+	if (tz)
+	{
+		if (!tzflag)
+		{
+			_tzset();
+			tzflag++;
+		}
+		tz->tz_minuteswest = _timezone / 60;
+		tz->tz_dsttime = _daylight;
+	}
+	
+	return 0;
+}
+
+#endif
+
+int aphex_event_timedwait( AphexEvent *event , int seconds)
+{
+    int t_ret = 0;
+
+#ifdef USE_PTHREAD
+    struct timespec   ts;
+    struct timeval    tp;
+
+    gettimeofday(&tp, NULL);
+    ts.tv_sec  = tp.tv_sec;
+    ts.tv_nsec = tp.tv_usec * 1000;
+    ts.tv_sec += seconds;
+
+    t_ret = pthread_cond_timedwait(&(event->cond), &(event->mutex), &ts);
+#endif
+    
+    if (t_ret != 0)
+        return 1;
+    else
+        return 0;
+}
