@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2002 Evan Webb, Chris Ross
+ * Copyright (C) 2001-2007 Evan Webb, Chris Ross
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,10 +36,6 @@
 #include <ctype.h>
 #include <string.h>
 
-#ifndef FERITE_FRAMEWORK
-#include "../config.h"
-#endif
-
 #ifdef USE_DLFCN
 # include <dlfcn.h>
 #endif 
