@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2002 Evan Webb, Chris Ross
+ * Copyright (C) 2001-2007 Evan Webb, Chris Ross
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,10 +27,6 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef WIN32
-# include "../../../config.h"
-#endif
-
 #include "triton.h"
 
 int triton_init(void)
