@@ -31,40 +31,37 @@
    as the lib loader
 */
 
+#ifdef WIN32
 
-#include "../config.h"
-
-#ifdef USE_WIN32
-
-#include <windows.h>
+# include <windows.h>
 # include "triton.h"
 
 LIB_INIT {
-   return 1;
+	return 0;
 }
 
 LIB_DEINIT {
-   return;
+	return;
 }
 
 LIB_EXT {
-   return ".dll";
+	return ".dll";
 }
 
 LIB_OPEN {
-   return LoadLibrary( path );
+	return LoadLibrary( path );
 }
 
 LIB_SYM {
-   return GetProcAddress( handle, symbol );
+	return GetProcAddress( handle, symbol );
 }
 
 LIB_CLOSE {
-   return FreeLibrary( handle );
+	return FreeLibrary( handle );
 }
 
 LIB_ERROR {
-   return "";
+	return "Win32 Library Loader Error";
 }
 
 #endif
