@@ -7,17 +7,17 @@ case "$1" in
   bootstrap)
     # generate the subdirs stuff for configure.in
     echo "Generating module list...";
-    echo "echo \"\"" > config.m4
-    echo "echo \"Configuring ferite modules...\"" >> config.m4
-    echo "echo \"\"" >> config.m4
-    echo "modules=\"\"" >> config.m4
+    echo "echo \"\"" > ../modules-config.m4
+    echo "echo \"Configuring ferite modules...\"" >> ../modules-config.m4
+    echo "echo \"\"" >> ../modules-config.m4
+    echo "modules=\"\"" >> ../modules-config.m4
     modulemakefiles=""
     for dir in *; do
       if test -d "$dir"; then
         cd $dir;
         if test -f "config.m4"; then
-	   echo "echo \"checking module $dir ...\"" >> ../config.m4
-	   echo "sinclude($moduledir/$dir/config.m4)" >> ../config.m4
+	   echo "echo \"checking module $dir ...\"" >> ../../modules-config.m4
+       echo "sinclude($moduledir/$dir/config.m4)" >> ../../modules-config.m4
 	   modulemakefiles="$modulemakefiles$moduledir\/$dir\/Makefile \\\\\\
 "
         fi
@@ -35,8 +35,8 @@ case "$1" in
         cd ..
       fi
     done
-    echo "AC_SUBST(modules)" >> config.m4
-    echo "echo \"\"" >> config.m4
+    echo "AC_SUBST(modules)" >> ../modules-config.m4
+    echo "echo \"\"" >> ../modules-config.m4
     sed -e "s/%modules%/$modulelist/" Makefile.am.stub > Makefile.am
     sed -e "s/%modules%/$modulemakefiles/g" ../configure.ac.stub > ../configure.ac
     ;;
@@ -83,7 +83,7 @@ case "$1" in
     dir=$2
 	if test -d "$dir"; then
 	  cd $dir;
-	  make maintainer-clean
+#	  make maintainer-clean
 	  if test -f "$dir.fec"; then
 		    ../../builder/builder -k -m $dir `echo \`for file in *.[ch]; do echo "-a $file "; done\` | egrep -v "^-a $dir" | sed -e "s/-a \*\.\[ch\]//"` `echo \`for file in *.fec; do echo "-A $file "; done\` | sed -e "s/-A \*\.fec//"` $dir.fec > /dev/null
 	  fi
