@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2003 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
+ * Copyright (C) 2004 Christian M. Stamgren
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -27,19 +28,75 @@
 #include "xml_header.h"
 #include "sax_handlers.h"
 
+#define DELIVER_STRING_CALLBACK( NAME, STRING, LENGTH ) \
+    do { \
+        SaxRecord             *sr = ctxt; \
+        FeriteObject          *obj = sr->obj; \
+        FeriteScript          *script = sr->script; \
+        FeriteFunction        *function = NULL; \
+        FeriteVariable        **params = NULL; \
+        FeriteString          *buf; \
+        if( (function = ferite_object_get_function( script, obj, NAME )) != NULL ) { \
+            buf = ferite_str_new(script,(char *)STRING, LENGTH, FE_CHARSET_DEFAULT); \
+            params = ferite_create_parameter_list_from_data( script, "s", buf ); \
+            ferite_variable_destroy( script, ferite_call_function( sr->script, obj, NULL, function, params ) ); \
+            ferite_delete_parameter_list( script, params ); \
+            ferite_str_destroy( script,buf ); \
+        } \
+    } while(0)
+#define DELIVER_2STRING_CALLBACK( NAME, STRING, LENGTH, STRING1, LENGTH1 ) \
+    do { \
+        SaxRecord             *sr = ctxt; \
+        FeriteObject          *obj = sr->obj; \
+        FeriteScript          *script = sr->script; \
+        FeriteFunction        *function = NULL; \
+        FeriteVariable        **params = NULL; \
+        FeriteString          *buf, *buf1; \
+        if( (function = ferite_object_get_function( script, obj, NAME )) != NULL ) { \
+            buf = ferite_str_new(script,(char *)STRING, LENGTH, FE_CHARSET_DEFAULT); \
+            buf1 = ferite_str_new(script,(char *)STRING1, LENGTH1, FE_CHARSET_DEFAULT); \
+            params = ferite_create_parameter_list_from_data( script, "ss", buf, buf1 ); \
+            ferite_variable_destroy( script, ferite_call_function( sr->script, obj, NULL, function, params ) ); \
+            ferite_delete_parameter_list( script, params ); \
+            ferite_str_destroy( script,buf ); \
+            ferite_str_destroy( script,buf1 ); \
+        } \
+    } while(0)
+#define DELIVER_3STRING_CALLBACK( NAME, STRING, LENGTH, STRING1, LENGTH1, STRING2, LENGTH2 ) \
+    do { \
+        SaxRecord             *sr = ctxt; \
+        FeriteObject          *obj = sr->obj; \
+        FeriteScript          *script = sr->script; \
+        FeriteFunction        *function = NULL; \
+        FeriteVariable        **params = NULL; \
+        FeriteString          *buf, *buf1, *buf2; \
+        if( (function = ferite_object_get_function( script, obj, NAME )) != NULL ) { \
+            buf = ferite_str_new(script,(char *)STRING, LENGTH, FE_CHARSET_DEFAULT); \
+            buf1 = ferite_str_new(script,(char *)STRING1, LENGTH1, FE_CHARSET_DEFAULT); \
+            buf2 = ferite_str_new(script,(char *)STRING2, LENGTH2, FE_CHARSET_DEFAULT); \
+            params = ferite_create_parameter_list_from_data( script, "sss", buf, buf1, buf2 ); \
+            ferite_variable_destroy( script, ferite_call_function( sr->script, obj, NULL, function, params ) ); \
+            ferite_delete_parameter_list( script, params ); \
+            ferite_str_destroy( script,buf ); \
+            ferite_str_destroy( script,buf1 ); \
+            ferite_str_destroy( script,buf2 ); \
+        } \
+    } while(0) 
+
 xmlParserInputPtr sax_resolveEntity(void *ctxt, const xmlChar *publicId, const xmlChar *systemId)
 {
     printf( "In Function %s - %s\n", __FUNCTION__, ((SaxRecord*)ctxt)->obj->name );
+    return NULL;
 }
 
 void sax_internalSubset(void *ctxt, const xmlChar *name, const xmlChar *ExternalID, const xmlChar *SystemID)
 {
-    printf( "In Function %s - %s\n", __FUNCTION__, ((SaxRecord*)ctxt)->obj->name );
+    DELIVER_3STRING_CALLBACK("internalSubset", name, 0, ExternalID, 0, SystemID, 0);
 }
 
 void sax_externalSubset (void *ctxt, const xmlChar *name, const xmlChar *ExternalID, const xmlChar *SystemID)
 {
-    printf( "In Function %s - %s\n", __FUNCTION__, ((SaxRecord*)ctxt)->obj->name );
+    DELIVER_3STRING_CALLBACK("externalSubset", name, 0, ExternalID, 0, SystemID, 0);
 }
 
 xmlEntityPtr sax_getEntity (void *ctxt, const xmlChar *name)
@@ -51,6 +108,7 @@ xmlEntityPtr sax_getEntity (void *ctxt, const xmlChar *name)
 xmlEntityPtr sax_getParameterEntity (void *ctxt, const xmlChar *name)
 {
     printf( "In Function %s - %s\n", __FUNCTION__, ((SaxRecord*)ctxt)->obj->name );
+    return NULL;
 }
 
 void sax_entityDecl (void *ctxt, const xmlChar *name, int type, const xmlChar *publicId, const xmlChar *systemId, xmlChar *content)
@@ -89,14 +147,9 @@ void sax_startDocument( void *ctxt )
     FeriteObject          *obj = sr->obj;
     FeriteScript          *script = sr->script;
     FeriteFunction        *function = NULL;
-    FeriteVariable        **params = NULL;
 
     if( (function = ferite_object_get_function( script, obj, "startDocument" )) != NULL )
-    {
-        params = ferite_create_parameter_list_from_data( script, "oo", obj, obj );
-        ferite_variable_destroy( script, ferite_call_function( sr->script, function, params ) );
-        ferite_delete_parameter_list( script, params );
-    }
+        ferite_variable_destroy( script, ferite_call_function( sr->script, obj, NULL, function, NULL ) );
 }
 
 void sax_endDocument( void *ctxt )
@@ -105,14 +158,9 @@ void sax_endDocument( void *ctxt )
     FeriteObject          *obj = sr->obj;
     FeriteScript          *script = sr->script;
     FeriteFunction        *function = NULL;
-    FeriteVariable        **params = NULL;
 
     if( (function = ferite_object_get_function( script, obj, "endDocument" )) != NULL )
-    {
-        params = ferite_create_parameter_list_from_data( script, "oo", obj, obj );
-        ferite_variable_destroy( script, ferite_call_function( sr->script, function, params ) );
-        ferite_delete_parameter_list( script, params );
-    }
+        ferite_variable_destroy( script, ferite_call_function( sr->script, obj, NULL, function, NULL ) );
 }
 
 void sax_startElement (void *ctxt, const xmlChar *name, const xmlChar **atts)
@@ -128,8 +176,8 @@ void sax_startElement (void *ctxt, const xmlChar *name, const xmlChar **atts)
 
     if( (function = ferite_object_get_function( script, obj, "startElement" )) != NULL )
     {
-        tname = ferite_str_new((char *)name, 0, FE_CHARSET_DEFAULT);
-        array = ferite_create_uarray_variable( NULL, "retval-string_split", 10, FE_STATIC );
+        tname = ferite_str_new(script, (char *)name, 0, FE_CHARSET_DEFAULT);
+        array = ferite_create_uarray_variable( script, "retval-string_split", 10, FE_STATIC );
         if( atts != NULL )
         {
             for( i = 0; atts[i] != NULL; i++ )
@@ -138,124 +186,52 @@ void sax_startElement (void *ctxt, const xmlChar *name, const xmlChar **atts)
                 ferite_uarray_add( script, VAUA( array ), var, (char *)atts[i-1], FE_ARRAY_ADD_AT_END );
             }
         }
-        params = ferite_create_parameter_list_from_data( script, "saoo", tname, VAUA(array), obj, obj );
-        ferite_variable_destroy( script, ferite_call_function( sr->script, function, params ) );
+        params = ferite_create_parameter_list_from_data( script, "sa", tname, VAUA(array) );
+        ferite_variable_destroy( script, ferite_call_function( sr->script, obj, NULL, function, params ) );
         ferite_delete_parameter_list( script, params );
         ferite_variable_destroy( script, array );
-        ferite_str_destroy( tname );
+        ferite_str_destroy( script, tname );
     }
 }
 
 void sax_endElement (void *ctxt, const xmlChar *name)
 {
-    SaxRecord             *sr = ctxt;
-    FeriteObject          *obj = sr->obj;
-    FeriteScript          *script = sr->script;
-    FeriteFunction        *function = NULL;
-    FeriteVariable        **params = NULL;
-    FeriteString          *tname;
-
-    if( (function = ferite_object_get_function( script, obj, "endElement" )) != NULL )
-    {
-        tname = ferite_str_new((char *)name, 0, FE_CHARSET_DEFAULT);
-        params = ferite_create_parameter_list_from_data( script, "soo", tname, obj, obj );
-        ferite_variable_destroy( script, ferite_call_function( sr->script, function, params ) );
-        ferite_delete_parameter_list( script, params );
-        ferite_str_destroy( tname );
-    }
+    DELIVER_STRING_CALLBACK("endElement", name, 0);
 }
 
 void sax_attribute (void *ctxt, const xmlChar *name, const xmlChar *value)
 {
-    printf( "In Function %s - %s\n", __FUNCTION__, ((SaxRecord*)ctxt)->obj->name );
+    DELIVER_2STRING_CALLBACK( "attribute", name, 0, value, 0 );
 }
 
 void sax_reference (void *ctxt, const xmlChar *name)
 {
-    printf( "In Function %s - %s\n", __FUNCTION__, ((SaxRecord*)ctxt)->obj->name );
+    DELIVER_STRING_CALLBACK( "reference", name, 0 );
 }
 
 void sax_characters (void *ctxt, const xmlChar *ch, int len)
 {
-    SaxRecord             *sr = ctxt;
-    FeriteObject          *obj = sr->obj;
-    FeriteScript          *script = sr->script;
-    FeriteFunction        *function = NULL;
-    FeriteVariable        **params = NULL;
-    FeriteString          *buf;
-
-    if( (function = ferite_object_get_function( script, obj, "characters" )) != NULL )
-    {
-        buf = ferite_str_new((char *)ch, len, FE_CHARSET_DEFAULT);
-        params = ferite_create_parameter_list_from_data( script, "soo", buf, obj, obj );
-        ferite_variable_destroy( script, ferite_call_function( sr->script, function, params ) );
-        ferite_delete_parameter_list( script, params );
-        ferite_str_destroy( buf );
-    }
+    DELIVER_STRING_CALLBACK( "characters", ch, len );
 }
 
 void sax_ignorableWhitespace (void *ctxt, const xmlChar *ch, int len)
 {
-    printf( "In Function %s - %s\n", __FUNCTION__, ((SaxRecord*)ctxt)->obj->name );
+    DELIVER_STRING_CALLBACK( "ignorableWhitespace", ch, len );
 }
 
 void sax_processingInstruction (void *ctxt, const xmlChar *target, const xmlChar *data)
 {
-    SaxRecord             *sr = ctxt;
-    FeriteObject          *obj = sr->obj;
-    FeriteScript          *script = sr->script;
-    FeriteFunction        *function = NULL;
-    FeriteVariable        **params = NULL;
-    FeriteString          *name,*value;
-
-    if( (function = ferite_object_get_function( script, obj, "pi" )) != NULL )
-    {
-        name = ferite_str_new((char *)target, 0, FE_CHARSET_DEFAULT);
-        value = ferite_str_new((char *)data, 0, FE_CHARSET_DEFAULT);
-        params = ferite_create_parameter_list_from_data( script, "ssoo", name, value, obj, obj );
-        ferite_variable_destroy( script, ferite_call_function( sr->script, function, params ) );
-        ferite_delete_parameter_list( script, params );
-        ferite_str_destroy( name );
-        ferite_str_destroy( value );
-    }
+    DELIVER_2STRING_CALLBACK( "processingInstruction", target, 0, data, 0 );
 }
 
 void sax_comment (void *ctxt, const xmlChar *value)
 {
-    SaxRecord             *sr = ctxt;
-    FeriteObject          *obj = sr->obj;
-    FeriteScript          *script = sr->script;
-    FeriteFunction        *function = NULL;
-    FeriteVariable        **params = NULL;
-    FeriteString          *buf;
-
-    if( (function = ferite_object_get_function( script, obj, "comment" )) != NULL )
-    {
-        buf = ferite_str_new((char *)value, 0, FE_CHARSET_DEFAULT);
-        params = ferite_create_parameter_list_from_data( script, "soo", buf, obj, obj );
-        ferite_variable_destroy( script, ferite_call_function( sr->script, function, params ) );
-        ferite_delete_parameter_list( script, params );
-        ferite_str_destroy( buf );
-    }
+    DELIVER_STRING_CALLBACK( "command", value, strlen((char*)value) );
 }
 
 void sax_cdataBlock (void *ctxt, const xmlChar *value, int len)
 {
-    SaxRecord             *sr = ctxt;
-    FeriteObject          *obj = sr->obj;
-    FeriteScript          *script = sr->script;
-    FeriteFunction        *function = NULL;
-    FeriteVariable        **params = NULL;
-    FeriteString          *buf;
-
-    if( (function = ferite_object_get_function( script, obj, "cdata" )) != NULL )
-    {
-        buf = ferite_str_new((char *)value, len, FE_CHARSET_DEFAULT);
-        params = ferite_create_parameter_list_from_data( script, "soo", buf, obj, obj );
-        ferite_variable_destroy( script, ferite_call_function( sr->script, function, params ) );
-        ferite_delete_parameter_list( script, params );
-        ferite_str_destroy( buf );
-    }
+    DELIVER_STRING_CALLBACK( "cdata", value, len );
 }
 
 void sax_warning (void *ctxt, const char *msg, ...)
@@ -274,7 +250,7 @@ void sax_error (void *ctxt, const char *msg, ...)
     va_list ap;
 
     va_start( ap, msg );
-    ferite_verror( sr->script, (char *)msg, &ap );
+    ferite_verror( sr->script, 0, (char *)msg, &ap );
     va_end( ap );
 }
 
@@ -291,16 +267,19 @@ void sax_fatalError (void *ctxt, const char *msg, ...)
 int sax_isStandalone (void *ctxt)
 {
     printf( "In Function %s - %s\n", __FUNCTION__, ((SaxRecord*)ctxt)->obj->name );
+	return FE_FALSE;
 }
 
 int sax_hasInternalSubset (void *ctxt)
 {
     printf( "In Function %s - %s\n", __FUNCTION__, ((SaxRecord*)ctxt)->obj->name );
+	return FE_FALSE;
 }
 
 int sax_hasExternalSubset (void *ctxt)
 {
     printf( "In Function %s - %s\n", __FUNCTION__, ((SaxRecord*)ctxt)->obj->name );
+	return FE_FALSE;
 }
 
 int sax_xmlParseFile( xmlSAXHandlerPtr sax, SaxRecord *obj, char *filename )
