@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2003 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
+ * Copyright (C) 2004 Christian M. Stamgren
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -31,17 +32,21 @@
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include <libxml/xmlerror.h>
+#include <libxml/parserInternals.h>
+#include <libxml/xpathInternals.h>
 #include "ferite.h"
 
 
 typedef struct __xmldoc {
    xmlDocPtr doc;
    xmlNodePtr node;
+   int keepBlanks;
 } XMLDoc;
 
 
-
-void tree_error_handler(void * ctxt, const char * msg, ...);
-
+void               tree_error_handler(void * ctxt, const char * msg, ...);
+FeriteVariable    *create_element_node( FeriteScript *script, xmlDocPtr doc, xmlNodePtr node );
+xmlXPathObjectPtr  get_nodes_with_name_xpath( XMLDoc *tree, FeriteString *str );
+void               recursive_namespace_copy( xmlNodePtr target, xmlNodePtr ref );
 
 #endif /* __TREE_HANDLERS_H__ */
