@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2003 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
+ * Copyright (C) 2004 Christian M. Stamgren
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -26,12 +27,11 @@
 
 FeriteVariable *ParseXPath( FeriteScript *script, XMLDoc *tree, const char *str )
 {
-    xmlXPathObjectPtr res;
-    xmlNodeSetPtr cur;
-    xmlXPathCompExprPtr comp;
-    xmlChar *string;
-    FeriteVariable *array,*element;
-    register int i;
+    xmlXPathObjectPtr res = NULL;
+    xmlNodeSetPtr cur = NULL;
+    xmlXPathCompExprPtr comp = NULL;
+    FeriteVariable *array = NULL,*element = NULL;
+    register int i = 0;
 
     xmlXPathContextPtr ctxt = xmlXPathNewContext( tree->doc );
     ctxt->node = tree->node;
@@ -56,10 +56,7 @@ FeriteVariable *ParseXPath( FeriteScript *script, XMLDoc *tree, const char *str
             cur = res->nodesetval;
             for (i = 0 ; i < cur->nodeNr ; i++)
             {
-                string = xmlXPathCastNodeToString(cur->nodeTab[i]);
-
-                element = fe_new_str_static("xpath_result", (char*)string, 0, FE_CHARSET_DEFAULT);
-                xmlFree( string );
+                element = create_element_node( script, tree->doc, cur->nodeTab[i] );		
                 ferite_uarray_add( script, VAUA( array ), element, NULL, FE_ARRAY_ADD_AT_END );
             }
             break;
@@ -94,7 +91,9 @@ FeriteVariable *ParseXPath( FeriteScript *script, XMLDoc *tree, const char *str
             break;
         }
     }
-    xmlXPathFreeObject(res);
+    if( res != NULL)
+      xmlXPathFreeObject(res);
+
     xmlXPathFreeContext(ctxt);
 
     return array;
