@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -28,6 +28,10 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#ifdef HAVE_CONFIG_HEADER
+#include "../config.h"
+#endif
+
 #include "ferite.h"
 #include "aphex.h"
 
@@ -47,10 +51,9 @@ void ferite_init_std_gc( FeriteScript *script )
     {
         gc = script->gc = fmalloc( sizeof(FeriteStdGC) );
         gc->size = FE_GC_INITIAL_SIZE;
-        gc->contents = fmalloc( sizeof(FeriteObject) * gc->size );
+        gc->contents = fmalloc( sizeof(FeriteObject*) * gc->size );
         for( i = 0; i < gc->size; i++ )
-          gc->contents[i] = NULL;
-        gc->count = 0;
+			gc->contents[i] = NULL;
     }
     FE_LEAVE_FUNCTION( NOWT );
 }
@@ -135,6 +138,7 @@ void ferite_check_std_gc( FeriteScript *script )
 
     FE_ENTER_FUNCTION;
     FE_ASSERT( script != NULL && script->gc != NULL );
+	script->gc_running = FE_TRUE;
 
     gc = script->gc;
     FUD(( "GC: Scanning gc for unreferenced objects\n" ));
@@ -147,6 +151,9 @@ void ferite_check_std_gc( FeriteScript *script )
             gc->contents[i] = NULL;
         }
     }
+	script->gc_count = 0;
+	
+	script->gc_running = FE_FALSE;
     FE_LEAVE_FUNCTION( NOWT );
 }
 void ferite_merge_std_gc( FeriteScript *script, void *g )
