@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -32,8 +32,8 @@
 #include <ferite/fstructs.h>
 
 /**
- * !group Engine Globals
- * !description This is where the main globals are held. These are used all 
+ * @group Engine Globals
+ * @description This is where the main globals are held. These are used all 
  *              over the ferite engine and provide useful no-area specific
  *              functions.
  */
@@ -42,9 +42,9 @@
 int                ferite_call_level = 0;
 
 /** 
- * !variable ferite_show_debug
- * !type int 
- * !brief A flag to say whether or not we should show debug output 
+ * @variable ferite_show_debug
+ * @type int 
+ * @brief A flag to say whether or not we should show debug output 
  */
 int                ferite_show_debug = 0;
 
@@ -60,71 +60,80 @@ void  (*ferite_memory_init)(void);
 void  (*ferite_memory_deinit)(void);
 
 /**
- * !function fmalloc
- * !declaration void *fmalloc( size_t size )
- * !brief Allocate a block of memory
- * !param size_t size The size of the memory required
- * !return A pointer to the allocated memory
+ * @function fmalloc
+ * @declaration void *fmalloc( size_t size )
+ * @brief Allocate a block of memory
+ * @param size_t size The size of the memory required
+ * @return A pointer to the allocated memory
  */
-void *(*ferite_malloc)( size_t size, char *name, int line );
+void *(*ferite_malloc)( size_t size, char *name, int line, FeriteScript *script );
 
 /**
- * !function fcalloc
- * !declaration fcalloc( size_t size, size_t blk_size )
- * !brief Allocate a block of memory and have it zero'd to blk_size blocks
- * !param size_t size The size of memory to allocate
- * !param size_t blk_size Size of blocks
- * !return A pointer to the allocated memory
+ * @function fcalloc
+ * @declaration fcalloc( size_t size, size_t blk_size )
+ * @brief Allocate a block of memory and have it zero'd to blk_size blocks
+ * @param size_t size The size of memory to allocate
+ * @param size_t blk_size Size of blocks
+ * @return A pointer to the allocated memory
  */
-void *(*ferite_calloc)( size_t size, size_t blk_size, char *name, int line );
+void *(*ferite_calloc)( size_t size, size_t blk_size, char *name, int line, FeriteScript *script );
 
 /**
- * !function frealloc
- * !declaration void *frealloc( void *ptr, size_t size )
- * !brief Reallocate a block of memory to be smaller or bigger
- * !param void *ptr The memory to resize
- * !param size_t size The new size of the memory block
- * !return The pointer to the block of memory
+ * @function frealloc
+ * @declaration void *frealloc( void *ptr, size_t size )
+ * @brief Reallocate a block of memory to be smaller or bigger
+ * @param void *ptr The memory to resize
+ * @param size_t size The new size of the memory block
+ * @return The pointer to the block of memory
  */
-void *(*ferite_realloc)( void *ptr, size_t size );
+void *(*ferite_realloc)( void *ptr, size_t size, FeriteScript *script );
 
 /**
- * !function ffree
- * !declaration void ffree( void *ptr );
- * !brief Free a block of memory
- * !param void *ptr The memory to free up
+ * @function ffree
+ * @declaration void ffree( void *ptr );
+ * @brief Free a block of memory
+ * @param void *ptr The memory to free up
  */
-void  (*ferite_free)( void *ptr, char *file, int line );
+void  (*ferite_free)( void *ptr, char *file, int line, FeriteScript *script );
 
 /* Function pointers for Garbage Collection */
 void  (*ferite_init_gc)( FeriteScript *script );
 void  (*ferite_deinit_gc)( FeriteScript *script );
 
 /**
- * !function ferite_add_to_gc
- * !declaration void ferite_add_to_gc( FeriteScript *script, FeriteObject *obj )
- * !brief Add an object to the garbage collector
- * !param FeriteScript *script The script
- * !param FeriteObject *obj The object to add
+ * @function ferite_add_to_gc
+ * @declaration void ferite_add_to_gc( FeriteScript *script, FeriteObject *obj )
+ * @brief Add an object to the garbage collector
+ * @param FeriteScript *script The script
+ * @param FeriteObject *obj The object to add
  */
 void  (*ferite_add_to_gc)( FeriteScript *script, FeriteObject *obj );
 
 /**
- * !function ferite_check_gc
- * !declaration void ferite_check_gc( FeriteScript *script )
- * !brief Cause a check of the garbage collector
- * !param FeriteScript *script The script
+ * @function ferite_check_gc
+ * @declaration void ferite_check_gc( FeriteScript *script )
+ * @brief Cause a check of the garbage collector
+ * @param FeriteScript *script The script
  */
 void  (*ferite_check_gc)( FeriteScript *script );
 void  (*ferite_merge_gc)( FeriteScript *script, void *gc );
 
 /**
- * !variable ferite_ARGV
- * !type FeriteVariable
- * !brief This is the arguments that can be passed to a script
+ * @variable ferite_ARGV
+ * @type FeriteVariable
+ * @brief This is the arguments that can be passed to a script
  */
 FeriteVariable *ferite_ARGV = NULL;
 
+jmp_buf ferite_exception_jmpback;
+int ferite_exception_status;
+
+void ferite_debug_catch( void *p, int count ) 
+{
+	fprintf( stderr, "ferite_debug_catch(%p,%d): Sleeping for 30 seconds (process id: %d)\n", p, count, getpid() );
+	sleep(30);
+}
+
 /**
- * !end
+ * @end
  */
