@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -28,11 +28,15 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#ifdef HAVE_CONFIG_HEADER
+#include "../config.h"
+#endif
+
 #include "ferite.h"
 
 /**
- * !group Hashes
- * !description How could any program survive without hashing? The hash functions
+ * @group Hashes
+ * @description How could any program survive without hashing? The hash functions
  *              below provide a good hashing system implementation which is used
  *              throughout the ferite engine. They can be used to store anything
  *              and form the basis for namespaces.
@@ -40,9 +44,17 @@
 
 extern int ferite_pow_lookup[];
 
-unsigned int ferite_hash_gen( char *key, int keylen )
+/**
+ * @function ferite_hash_gen
+ * @declaration unsigned int ferite_hash_gen( char *key, size_t keylen )
+ * @brief Generate a hash value given a key and a hash length
+ * @param char *key The value to hash
+ * @param size_t keylen The size of the hash
+ * @return An unsigned integer value representing the hash value
+ */
+unsigned int ferite_hash_gen( char *key, size_t keylen )
 {
-    int i;
+    size_t i;
     unsigned int hashval = 0;
 
     FE_ENTER_FUNCTION;
@@ -54,7 +66,7 @@ unsigned int ferite_hash_gen( char *key, int keylen )
 FeriteHashBucket *ferite_create_hash_bucket( FeriteScript *script, char *key, void *value )
 {
     FeriteHashBucket *ptr;
-    int len;
+    size_t len;
 
     FE_ENTER_FUNCTION;
     len = strlen( key );
@@ -68,12 +80,12 @@ FeriteHashBucket *ferite_create_hash_bucket( FeriteScript *script, char *key, vo
 }
 
 /**
- * !function ferite_create_hash
- * !declaration FeriteHash *ferite_create_hash( FeriteScript *script, int size )
- * !brief Create a hash table
- * !param FeriteScript *script The script
- * !param int size The initial size of the hash
- * !return A new hash table
+ * @function ferite_create_hash
+ * @declaration FeriteHash *ferite_create_hash( FeriteScript *script, int size )
+ * @brief Create a hash table
+ * @param FeriteScript *script The script
+ * @param int size The initial size of the hash
+ * @return A new hash table
  */
 FeriteHash *ferite_create_hash( FeriteScript *script, int size )
 {
@@ -84,37 +96,39 @@ FeriteHash *ferite_create_hash( FeriteScript *script, int size )
     while( size > ferite_pow_lookup[i] )
       i++;
     size = ferite_pow_lookup[i];
-    ptr = fcalloc( 1, sizeof( FeriteHash ) + ( size * sizeof( FeriteHashBucket ) ) );
+    ptr = fcalloc( 1, sizeof( FeriteHash ) + ( size * sizeof(FeriteHashBucket*) ) );
     ptr->size = size;
     ptr->hash = (void*)((char *)ptr + sizeof( FeriteHash ));
+	ptr->count = 0;
     FE_LEAVE_FUNCTION( ptr );
 }
 
 /**
- * !function ferite_delete_hash
- * !declaration void ferite_delete_hash( FeriteScript *script, FeriteHash *hash, void (*cb)(FeriteScript*,void *data) )
- * !brief Delete a hash table
- * !param FeriteScript *script The script
- * !param FeriteHash *hash The hash table to delete
- * !param void cb The function to call to delete the hash's contents
+ * @function ferite_delete_hash
+ * @declaration void ferite_delete_hash( FeriteScript *script, FeriteHash *hash, void (*cb)(FeriteScript*,void *data) )
+ * @brief Delete a hash table
+ * @param FeriteScript *script The script
+ * @param FeriteHash *hash The hash table to delete
+ * @param void cb The function to call to delete the hash's contents
  */
 void ferite_delete_hash( FeriteScript *script, FeriteHash *hash, void (*cb)(FeriteScript*,void *data) )
 {
-    int i;
-    FeriteHashBucket *bucket, *next;
+    int i = 0;
+    FeriteHashBucket *bucket = NULL;
 
     FE_ENTER_FUNCTION;
     FE_ASSERT( hash != NULL );
     if( hash->hash != NULL )
     {
-        for( i = 0; i < hash->size; i++ )
-        {
-            for( bucket = hash->hash[i]; bucket; bucket = next )
-            {
-                next = bucket->next;
+        for( i = 0; i < hash->size; i++ ) {
+			int chain_count = 0;
+            for( bucket = hash->hash[i]; bucket; bucket = bucket->next ) {
+                FeriteHashBucket *current = bucket;
+				chain_count++;
+				FUD(("[%p] Index: %d, Chain: %d, Data: %p\n", hash, i, chain_count, current));
                 if( cb )
-                  (cb)( script, bucket->data );
-                ffree( bucket );
+                  (cb)( script, current->data );
+                ffree( current );
             }
         }
     }
@@ -124,12 +138,12 @@ void ferite_delete_hash( FeriteScript *script, FeriteHash *hash, void (*cb)(Feri
 }
 
 /**
- * !function ferite_process_hash
- * !declaration void ferite_process_hash( FeriteScript *script, FeriteHash *hash, void (*cb)(FeriteScript*,void *data) )
- * !brief Process a hash table by calling a function on each element
- * !param FeriteScript *script The script
- * !param FeriteHash *hash The hash table to process
- * !param void cb The function to call to process the hash's contents
+ * @function ferite_process_hash
+ * @declaration void ferite_process_hash( FeriteScript *script, FeriteHash *hash, void (*cb)(FeriteScript*,void *data) )
+ * @brief Process a hash table by calling a function on each element
+ * @param FeriteScript *script The script
+ * @param FeriteHash *hash The hash table to process
+ * @param void cb The function to call to process the hash's contents
  */
 void ferite_process_hash( FeriteScript *script, FeriteHash *hash, void (*cb)(FeriteScript*,void *data,char *key) )
 {
@@ -151,13 +165,13 @@ void ferite_process_hash( FeriteScript *script, FeriteHash *hash, void (*cb)(Fer
 }
 
 /**
- * !function ferite_hash_add
- * !declaration void ferite_hash_add( FeriteScript *script, FeriteHash *hash, char *key, void *data )
- * !brief Add data to the hash
- * !param FeriteScript *script The script
- * !param FeriteHash *hash The hash to insert the data into
- * !param char *key The key on which to hash the data
- * !param void *data A pointer to some data to keep
+ * @function ferite_hash_add
+ * @declaration void ferite_hash_add( FeriteScript *script, FeriteHash *hash, char *key, void *data )
+ * @brief Add data to the hash
+ * @param FeriteScript *script The script
+ * @param FeriteHash *hash The hash to insert the data into
+ * @param char *key The key on which to hash the data
+ * @param void *data A pointer to some data to keep
  */
 void ferite_hash_add( FeriteScript *script, FeriteHash *hash, char *key, void *data )
 {
@@ -175,17 +189,18 @@ void ferite_hash_add( FeriteScript *script, FeriteHash *hash, char *key, void *d
     ptr = ferite_create_hash_bucket( script, key, data );
     ptr->next = hash->hash[loc];
     hash->hash[loc] = ptr;
+	hash->count++;
     FE_LEAVE_FUNCTION( NOWT );
 }
 
 /**
- * !function ferite_hash_get
- * !declaration void *ferite_hash_get( FeriteScript *script, FeriteHash *hash, char *key )
- * !brief Get the data at a specified key
- * !param FeriteScript *script The script to pass around
- * !param FeriteHash *hash The hash to get the data from
- * !param char *key The key to obtain the data for
- * !return The pointer to the data or NULL otherwise
+ * @function ferite_hash_get
+ * @declaration void *ferite_hash_get( FeriteScript *script, FeriteHash *hash, char *key )
+ * @brief Get the data at a specified key
+ * @param FeriteScript *script The script to pass around
+ * @param FeriteHash *hash The hash to get the data from
+ * @param char *key The key to obtain the data for
+ * @return The pointer to the data or NULL otherwise
  */
 void *ferite_hash_get( FeriteScript *script, FeriteHash *hash, char *key )
 {
@@ -213,11 +228,11 @@ void *ferite_hash_get( FeriteScript *script, FeriteHash *hash, char *key )
 }
 
 /**
- * !function ferite_create_iterator
- * !declaration FeriteIterator *ferite_create_iterator( FeriteScript *script )
- * !brief Create an iterator to iterate over a hash
- * !param FeriteScript *script The script
- * !return A pointer to an iterator
+ * @function ferite_create_iterator
+ * @declaration FeriteIterator *ferite_create_iterator( FeriteScript *script )
+ * @brief Create an iterator to iterate over a hash
+ * @param FeriteScript *script The script
+ * @return A pointer to an iterator
  */
 FeriteIterator *ferite_create_iterator( FeriteScript *script )
 {
@@ -231,14 +246,14 @@ FeriteIterator *ferite_create_iterator( FeriteScript *script )
 }
 
 /**
- * !function ferite_hash_walk
- * !declaration FeriteHashBucket *ferite_hash_walk(FeriteScript *script, FeriteHash *hash, FeriteIterator *iter)
- * !brief Walk the hash
- * !param FeriteScript *script The script
- * !param FeriteHash *hash The hash to walk over
- * !param FeriteIterator *iter The iterator to use
- * !description Each time this function is called it will return the next data value
- * !return The next hash bucket, NULL if there are no more
+ * @function ferite_hash_walk
+ * @declaration FeriteHashBucket *ferite_hash_walk(FeriteScript *script, FeriteHash *hash, FeriteIterator *iter)
+ * @brief Walk the hash
+ * @param FeriteScript *script The script
+ * @param FeriteHash *hash The hash to walk over
+ * @param FeriteIterator *iter The iterator to use
+ * @description Each time this function is called it will return the next data value
+ * @return The next hash bucket, NULL if there are no more
  */
 FeriteHashBucket *ferite_hash_walk(FeriteScript *script, FeriteHash *hash, FeriteIterator *iter)
 {
@@ -286,15 +301,15 @@ FeriteHashBucket *ferite_hash_walk(FeriteScript *script, FeriteHash *hash, Ferit
 }
 
 /**
- * !function ferite_hash_update
- * !declaration void ferite_hash_update( FeriteScript *script, FeriteHash *hash, char *key, char *data )
- * !brief Update a value at the specified hash key
- * !param FeriteScript *script The script
- * !param FeriteHash *hash The has to update
- * !param char *key The key to the bucket to update
- * !param char *data Pointer to the data
+ * @function ferite_hash_update
+ * @declaration void ferite_hash_update( FeriteScript *script, FeriteHash *hash, char *key, char *data )
+ * @brief Update a value at the specified hash key
+ * @param FeriteScript *script The script
+ * @param FeriteHash *hash The has to update
+ * @param char *key The key to the bucket to update
+ * @param char *data Pointer to the data
  */
-void ferite_hash_update( FeriteScript *script, FeriteHash *hash, char *key, char *data )
+void ferite_hash_update( FeriteScript *script, FeriteHash *hash, char *key, void *data )
 {
     int loc;
     unsigned int hashval;
@@ -320,22 +335,22 @@ void ferite_hash_update( FeriteScript *script, FeriteHash *hash, char *key, char
 }
 
 /**
- * !function ferite_hash_delete
- * !declaration void ferite_hash_delete( FeriteScript *script, FeriteHash *hash, char *key )
- * !brief Delete a key from the hash
- * !param FeriteScript *script The script to delete
- * !param FeriteHash *hash The hash to delete from 
- * !param char *key The key to delete
+ * @function ferite_hash_delete
+ * @declaration void ferite_hash_delete( FeriteScript *script, FeriteHash *hash, char *key )
+ * @brief Delete a key from the hash
+ * @param FeriteScript *script The script to delete
+ * @param FeriteHash *hash The hash to delete from 
+ * @param char *key The key to delete
  */
 void ferite_hash_delete( FeriteScript *script, FeriteHash *hash, char *key )
 {
-    int loc;
-    unsigned int hashval;
-    FeriteHashBucket *ptr;
-    FeriteHashBucket *optr;
+    int loc = 0;
+    unsigned int hashval = 0;
+    FeriteHashBucket *ptr = NULL;
+    FeriteHashBucket *optr = NULL;
 
     FE_ENTER_FUNCTION;
-    FE_ASSERT( hash != NULL  && key != NULL );
+    FE_ASSERT( hash != NULL && key != NULL );
 
     hashval = ferite_hash_gen( key, strlen( key ) );
     loc = hashval & hash->size - 1;
@@ -345,11 +360,12 @@ void ferite_hash_delete( FeriteScript *script, FeriteHash *hash, char *key )
         if( ptr->hashval == hashval && strcmp( key, ptr->id ) == 0 )
         {
             if( hash->hash[loc] == ptr )
-              hash->hash[loc] = ptr->next; /* remove head from array */
+                hash->hash[loc] = ptr->next; /* remove head from array */
             else
-              optr->next = ptr->next; /* remove buxket from list */
+                optr->next = ptr->next; /* remove buxket from list */
             FUD(("HASH: located data\n"));
             ffree( ptr );
+			hash->count--;
             FE_LEAVE_FUNCTION( NOWT );
         }
     }
@@ -358,13 +374,13 @@ void ferite_hash_delete( FeriteScript *script, FeriteHash *hash, char *key )
 }
 
 /**
- * !function ferite_hash_dup
- * !declaration FeriteHash *ferite_hash_dup( FeriteScript *script, FeriteHash *hash, void *(*ddup)( FeriteScript*,void *data,void *data2 ), void *data2 )
- * !brief Duplicate a hash
- * !param FeriteScript *script The script
- * !param FeriteHash *hash The hash table to duplicate
- * !param void *ddup The function that gets called with to duplicate a buckets data
- * !return A new hash that is an exact copy of the original
+ * @function ferite_hash_dup
+ * @declaration FeriteHash *ferite_hash_dup( FeriteScript *script, FeriteHash *hash, void *(*ddup)( FeriteScript*,void *data,void *data2 ), void *data2 )
+ * @brief Duplicate a hash
+ * @param FeriteScript *script The script
+ * @param FeriteHash *hash The hash table to duplicate
+ * @param void *ddup The function that gets called with to duplicate a buckets data
+ * @return A new hash that is an exact copy of the original
  */
 FeriteHash *ferite_hash_dup( FeriteScript *script, FeriteHash *hash, void *(*ddup)( FeriteScript*,void *data,void *data2 ), void *data2 )
 {
@@ -386,16 +402,17 @@ FeriteHash *ferite_hash_dup( FeriteScript *script, FeriteHash *hash, void *(*ddu
             bptr = bptr->next;
         }
     }
+	ptr->count = hash->count;
     FE_LEAVE_FUNCTION( ptr );
 }
 
 /**
- * !function ferite_hash_grow
- * !declaration FeriteHash *ferite_hash_grow( FeriteScript *script, FeriteHash *hash )
- * !brief Grow the hash table to be faster
- * !param FeriteScript *script The script
- * !param FeriteHash *hash The hash to modify
- * !return The pointer to the new hash (this may be the same as the passed in hash)
+ * @function ferite_hash_grow
+ * @declaration FeriteHash *ferite_hash_grow( FeriteScript *script, FeriteHash *hash )
+ * @brief Grow the hash table to be faster
+ * @param FeriteScript *script The script
+ * @param FeriteHash *hash The hash to modify
+ * @return The pointer to the new hash (this may be the same as the passed in hash)
  */
 FeriteHash *ferite_hash_grow( FeriteScript *script, FeriteHash *hash )
 {
@@ -427,12 +444,12 @@ FeriteHash *ferite_hash_grow( FeriteScript *script, FeriteHash *hash )
 }
 
 /**
- * !function ferite_hash_print
- * !declaration void ferite_hash_print( FeriteScript *script, FeriteHash *hash )
- * !brief Print out the hash table if debug mode is turned on
- * !param FeriteScript *script The script to delete
- * !param FeriteHash *hash The has to print out
- * !description This simply prints out what keys are in the hash for debug purposes
+ * @function ferite_hash_print
+ * @declaration void ferite_hash_print( FeriteScript *script, FeriteHash *hash )
+ * @brief Print out the hash table if debug mode is turned on
+ * @param FeriteScript *script The script to delete
+ * @param FeriteHash *hash The has to print out
+ * @description This simply prints out what keys are in the hash for debug purposes
  */
 void ferite_hash_print( FeriteScript *script, FeriteHash *hash )
 {
@@ -441,7 +458,7 @@ void ferite_hash_print( FeriteScript *script, FeriteHash *hash )
 
     FE_ENTER_FUNCTION;
     FE_ASSERT( hash != NULL );
-    FUD(("hash table-------------------------------\nsize: %d\n", hash->size));
+	FUD(("hash table-------------------------------\nsize: %d; count: %d\n", hash->size, hash->count));
     for( i = 0; i < hash->size; i++ )
     {
         FUD(("hash->hash[%d]:", i));
@@ -462,5 +479,5 @@ void ferite_hash_print( FeriteScript *script, FeriteHash *hash )
 }
 
 /**
- * !end
+ * @end
  */
