@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -28,13 +28,20 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#ifdef HAVE_CONFIG_HEADER
+#include "../config.h"
+#endif
+
 #include "ferite.h"
 #include "aphex.h"
 
+void ferite_classic_memory_dump();
+
 /*
  * this is to get huge amounts (well a fair amount) of debug info....
  * #define SUPER_VERBOSE
 */
+#undef FERITE_MEM_DEBUG
 
 int ferite_classic_malloc_count = 0;
 int ferite_classic_calloc_count = 0;
@@ -63,6 +70,13 @@ struct ferite_memory_block *mem_rootblk = NULL;
 void  ferite_classic_memory_init(void)
 {
     FE_ENTER_FUNCTION;
+	ferite_memory_init = ferite_classic_memory_init;
+	ferite_memory_deinit = ferite_classic_memory_deinit;
+	ferite_malloc = ferite_classic_malloc;
+	ferite_calloc = ferite_classic_calloc;
+	ferite_realloc = ferite_classic_realloc;
+	ferite_free = ferite_classic_free;
+
 #ifdef FERITE_MEM_DEBUG
     FUD(("Internal Memory Deubgging On.\n"));
     mem_rootblk = malloc(sizeof( struct ferite_memory_block ));
@@ -87,13 +101,13 @@ void ferite_classic_memory_clear_block( struct ferite_memory_block *ptr )
     if( ptr != NULL )
     {
         if( ptr->next )
-          ferite_classic_memory_clear_block( ptr->next );
+            ferite_classic_memory_clear_block( ptr->next );
         if( !ferite_hide_mem_use && ptr->line != 0 )
-          printf( "Freeing block[%p] [%d bytes] allocated on file: %s, line %d\n", ptr->data, ptr->size, ptr->file, ptr->line );
+            printf( "Freeing block[%p] [%d bytes] allocated on file: %s, line %d\n", ptr->data, ptr->size, ptr->file, ptr->line );
         ferite_total_freeup += ptr->size;
-      /* this is just here so i can see what string it is allocated */
-        if( ptr->line == 198 )
-          printf( "%s\n", (char *)ptr->data );
+        /* this is just here so i can see what string it is allocated */
+        if( ptr->line == 198 || ptr->line == 66 || ptr->line == 54 )
+            printf( "%s\n", (char *)ptr->data );
         free( ptr->data );
         free( ptr->file );
         free( ptr );
@@ -115,10 +129,10 @@ void ferite_classic_memory_deinit(void)
     }
 #ifdef FERITE_MEM_DEBUG
     if( !ferite_hide_mem_use )
-      printf("Freeeing Up Unallocated Memory\n");
+        printf("Freeeing Up Unallocated Memory\n");
     ferite_classic_memory_clear_block( mem_rootblk );
     if( !ferite_hide_mem_use )
-      printf("Free'd %d [%d bytes] allocations\n", ferite_classic_end_freeup, ferite_total_freeup );
+        printf("Free'd %d [%d bytes] allocations\n", ferite_classic_end_freeup, ferite_total_freeup );
 #endif
 #ifdef THREAD_SAFE
     aphex_mutex_destroy( ferite_classic_memory_lock );
@@ -126,18 +140,18 @@ void ferite_classic_memory_deinit(void)
     FE_LEAVE_FUNCTION( NOWT );
 }
 
-void *ferite_classic_malloc( size_t size, char *file, int line )
+void *ferite_classic_malloc( size_t size, char *file, int line, FeriteScript *script )
 {
 #ifdef FERITE_MEM_DEBUG
     struct ferite_memory_block *ptr;
 #endif
-
+    
     LOCK_MEMORY;
     ferite_classic_malloc_count++;
-
+    
 #ifdef FERITE_MEM_DEBUG
     for( ptr = mem_rootblk; ptr->next != NULL; ptr = ptr->next )
-      ;
+        ;
     ptr->next = malloc(sizeof( struct ferite_memory_block ));
     ptr->next->data = rmalloc( size );
     ptr->next->size = size;
@@ -158,25 +172,25 @@ void *ferite_classic_malloc( size_t size, char *file, int line )
 #endif
 }
 
-void *ferite_classic_calloc( size_t size, size_t blk_size, char *file, int line )
+void *ferite_classic_calloc( size_t size, size_t blk_size, char *file, int line, FeriteScript *script )
 {
 #ifdef FERITE_MEM_DEBUG
     struct ferite_memory_block *ptr;
 #endif
-
+    
     LOCK_MEMORY;
     ferite_classic_calloc_count++;
-
+    
 #ifdef FERITE_MEM_DEBUG
     for( ptr = mem_rootblk; ptr->next != NULL; ptr = ptr->next )
-      ;
+        ;
     ptr->next = malloc(sizeof( struct ferite_memory_block ));
     ptr->next->data = rcalloc( size, blk_size );
     ptr->next->size = size * blk_size;
     ptr->next->file = strdup( file );
     ptr->next->line = line;
     if( !ptr->next->data )
-      FUD(("ERROR: Unable to (c)allocate %d bytes.\n", size * blk_size));
+        FUD(("ERROR: Unable to (c)allocate %d bytes.\n", size * blk_size));
     ptr->next->next = NULL;
     UNLOCK_MEMORY;
     return ptr->next->data;
@@ -186,21 +200,19 @@ void *ferite_classic_calloc( size_t size, size_t blk_size, char *file, int line
 #endif
 }
 
-void *ferite_classic_realloc( void *targetptr, size_t size )
+void *ferite_classic_realloc( void *targetptr, size_t size, FeriteScript *script )
 {
 #ifdef FERITE_MEM_DEBUG
     struct ferite_memory_block *ptr;
 #endif
-
+    
     if( targetptr == NULL )
-    {
-        return ferite_classic_malloc( size, "ferite_mem.c", 125 );
-    }
+        return ferite_classic_malloc( size, "ferite_mem.c", 125, script );
     else
     {
         LOCK_MEMORY;
         ferite_classic_realloc_count++;
-
+        
 #ifdef FERITE_MEM_DEBUG
         for( ptr = mem_rootblk; ptr->next != NULL; ptr = ptr->next )
         {
@@ -219,14 +231,14 @@ void *ferite_classic_realloc( void *targetptr, size_t size )
     }
 }
 
-void ferite_classic_free( void *targetptr, char *file, int line )
+void ferite_classic_free( void *targetptr, char *file, int line, FeriteScript *script )
 {
 #ifdef FERITE_MEM_DEBUG
     struct ferite_memory_block *ptr, *temp_ptr;
-
+    
     if( targetptr == NULL )
-      printf( "Trying to free NULL ptr on %s:%d\n", file, line );
-
+        printf( "Trying to free NULL ptr on %s:%d\n", file, line );
+    
     LOCK_MEMORY;
     for( ptr = mem_rootblk; ptr->next != NULL; ptr = ptr->next )
     {
@@ -236,7 +248,7 @@ void ferite_classic_free( void *targetptr, char *file, int line )
             FUD(("INFO: Found memory block (%p). Freeing.\n", ptr->next->data));
 # endif
             ferite_classic_free_count++;
-
+            
             temp_ptr = ptr->next;
             ptr->next = temp_ptr->next;
 # ifdef SUPER_VERBOSE
@@ -254,7 +266,7 @@ void ferite_classic_free( void *targetptr, char *file, int line )
 # ifdef SUPER_VERBOSE
             FUD(("  * Free'd memory block\n"));
 # endif
-
+            
             UNLOCK_MEMORY;
             return;
         }
@@ -265,9 +277,9 @@ void ferite_classic_free( void *targetptr, char *file, int line )
 #else
     LOCK_MEMORY;
     if( targetptr == NULL )
-      printf( "Trying to free null in %s at %d\n", file, line );
+        printf( "Trying to free null in %s at %d\n", file, line );
     else
-      ferite_classic_free_count++;
+        ferite_classic_free_count++;
     rfree( targetptr );
     UNLOCK_MEMORY;
 #endif
@@ -277,7 +289,7 @@ void ferite_classic_memory_dump()
 {
     long memory_allocated = 0;
     struct ferite_memory_block *ptr;
-
+    
     if( mem_rootblk != NULL && mem_rootblk->next != NULL )
     {
         fprintf( stderr, "Currently Allocated Memory:\n" );
