@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -28,6 +28,10 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#ifdef HAVE_CONFIG_HEADER
+#include "../config.h"
+#endif
+
 #include "ferite.h"
 
 FeriteOp *ferite_create_op()
@@ -35,12 +39,13 @@ FeriteOp *ferite_create_op()
     FeriteOp *ptr;
 
     FE_ENTER_FUNCTION;
-    ptr = fmalloc( sizeof( FeriteOp ) );
+    ptr = fmalloc_ngc( sizeof( FeriteOp ) );
     ptr->OP_TYPE = F_OP_NOP;
     ptr->opdata = NULL;
     ptr->opdataf = NULL;
     ptr->line = 0;
     ptr->addr = 0;
+	ptr->flags = 0;
     FE_LEAVE_FUNCTION( ptr );
 }
 
@@ -49,11 +54,11 @@ FeriteOpcodeList *ferite_create_opcode_list( int size )
     FeriteOpcodeList *ptr;
 
     FE_ENTER_FUNCTION;
-    ptr = fmalloc( sizeof( FeriteOpcodeList ) );
+    ptr = fmalloc_ngc( sizeof( FeriteOpcodeList ) );
     ptr->size = size;
     ptr->filename = NULL;
     ptr->current_op_loc = -1;
-    ptr->list = fmalloc( sizeof( FeriteOp * ) * size );
+    ptr->list = fmalloc_ngc( sizeof( FeriteOp * ) * size );
     ptr->list[0] = NULL;
     FE_LEAVE_FUNCTION( ptr );
 }
@@ -70,7 +75,7 @@ void ferite_oplist_grow( FeriteOpcodeList *oplist )
 	 * - double oplist->size
 	 * - realloc */
         oplist->size *= 2;
-        oplist->list = frealloc( oplist->list, sizeof(FeriteOp *) * oplist->size );
+        oplist->list = frealloc_ngc( oplist->list, sizeof(FeriteOp *) * oplist->size );
         oplist->list[oplist->current_op_loc+1] = NULL;
     }
     FE_LEAVE_FUNCTION( NOWT );
@@ -97,8 +102,7 @@ FeriteOp *ferite_current_op( FeriteOpcodeList *oplist )
     FE_LEAVE_FUNCTION( oplist->list[oplist->current_op_loc] );
 }
 
-/* returns the address of the next operator with out incrmenting the internal
- * program counter */
+/* returns the address of the next operator with out incrmenting the internal program counter */
 FeriteOp *ferite_get_next_op_address( FeriteOpcodeList *oplist )
 {
     FE_ENTER_FUNCTION;
@@ -111,8 +115,7 @@ FeriteOp *ferite_get_next_op_address( FeriteOpcodeList *oplist )
     FE_LEAVE_FUNCTION( oplist->list[oplist->current_op_loc+1] );
 }
 
-/* returns the index of the next operator with out incrmenting the internal
- * program counter */
+/* returns the index of the next operator with out incrmenting the internal program counter */
 int ferite_get_next_op_loc( FeriteOpcodeList *oplist )
 {
     FE_ENTER_FUNCTION;
@@ -161,10 +164,6 @@ void ferite_delete_opcode_list( FeriteScript *script, FeriteOpcodeList *oplist )
                     ferite_variable_destroy( script, ptr );
                     freed_variables[currentVar] = ptr;
                 }
-                else
-                {
-                    printf( "CAN'T FREE DUPLICATE REFERENCE\n" );
-                }
             }
             ffree( oplist->list[i] );
             break;
@@ -187,14 +186,18 @@ void ferite_delete_opcode_list( FeriteScript *script, FeriteOpcodeList *oplist )
           case F_OP_METHOD:
           case F_OP_PUSHVAR:
           case F_OP_PUSHATTR:
-          case F_OP_MANY:
+		  case F_OP_DELIVER:
+		  case F_OP_CLSRE_ASSGN:
             if( oplist->list[i]->opdataf != NULL )
               ffree( oplist->list[i]->opdataf );
-            ffree( oplist->list[i]->opdata );
+			if( oplist->list[i]->opdata != NULL )
+	            ffree( oplist->list[i]->opdata );
             ffree( oplist->list[i] );
             break;
-          case F_OP_RGX:
-            ferite_delete_regex( oplist->list[i]->opdata );
+		  case F_OP_PUSHGLOBAL:
+          case F_OP_MANY:
+			if( oplist->list[i]->opdata != NULL )
+	            ffree( oplist->list[i]->opdata );
             ffree( oplist->list[i] );
             break;
           default:
@@ -220,8 +223,14 @@ void ferite_opcode_dump( FeriteOpcodeList *oplist )
     {
         switch( oplist->list[i]->OP_TYPE )
         {
-          case F_OP_PUSH:
-            printf( "[%d]\t [%p] PUSH        %s\n", i, (void *)(oplist->list[i]), ((FeriteVariable *)oplist->list[i]->opdata)->name );
+		  case -1: {
+			
+		  }
+          case F_OP_PUSH: {
+				FeriteString *str = ferite_variable_to_str( NULL, oplist->list[i]->opdata, FE_TRUE );
+	            printf( "[%d]\t [%p] PUSH         %s\n", i, (void *)(oplist->list[i]), str->data );
+				ferite_str_destroy( NULL, str );
+			}
             break;
           case F_OP_POP:
             printf( "[%d]\t [%p] POP\n", i, (void *)(oplist->list[i]) );
@@ -230,28 +239,28 @@ void ferite_opcode_dump( FeriteOpcodeList *oplist )
             printf( "[%d]\t [%p] NOP\n", i, (void *)(oplist->list[i]) );
             break;
           case F_OP_UNARY:
-            printf( "[%d]\t [%p] UNARYOP     %s\n", i, (void *)(oplist->list[i]), ferite_op_table[oplist->list[i]->addr].name );
+            printf( "[%d]\t [%p] UNARYOP      %s\n", i, (void *)(oplist->list[i]), ferite_op_table[oplist->list[i]->addr].name );
             break;
           case F_OP_BINARY:
-            printf( "[%d]\t [%p] BINARYOP    %s\n", i, (void *)(oplist->list[i]), ferite_op_table[oplist->list[i]->addr].name );
+            printf( "[%d]\t [%p] BINARYOP     %s\n", i, (void *)(oplist->list[i]), ferite_op_table[oplist->list[i]->addr].name );
             break;
           case F_OP_MANY:
-            printf( "[%d]\t [%p] MANYOP      %s\n", i, (void *)(oplist->list[i]), ferite_op_table[oplist->list[i]->addr].name );
+            printf( "[%d]\t [%p] MANYOP       %s\n", i, (void *)(oplist->list[i]), ferite_op_table[oplist->list[i]->addr].name );
             break;
           case F_OP_FUNCTION:
-            printf( "[%d]\t [%p] FUNCTION    %s\n", i, (void *)(oplist->list[i]), (char *)oplist->list[i]->opdata );
+            printf( "[%d]\t [%p] FUNCTION     %s\n", i, (void *)(oplist->list[i]), (char *)oplist->list[i]->opdata );
             break;
           case F_OP_METHOD:
-            printf( "[%d]\t [%p] METHOD      %s\n", i, (void *)(oplist->list[i]), (char *)oplist->list[i]->opdata );
+            printf( "[%d]\t [%p] METHOD       %s\n", i, (void *)(oplist->list[i]), (char *)oplist->list[i]->opdata );
             break;
           case F_OP_JMP:
-            printf( "[%d]\t [%p] JMP         %ld\n", i, (void *)(oplist->list[i]), oplist->list[i]->addr );
+            printf( "[%d]\t [%p] JMP          %ld\n", i, (void *)(oplist->list[i]), oplist->list[i]->addr );
             break;
           case F_OP_BNE:
-            printf( "[%d]\t [%p] BNE         %ld\n", i, (void *)(oplist->list[i]), oplist->list[i]->addr );
+            printf( "[%d]\t [%p] BNE          %ld\n", i, (void *)(oplist->list[i]), oplist->list[i]->addr );
             break;
           case F_OP_BIE:
-            printf( "[%d]\t [%p] BIE         %ld\n", i, (void *)(oplist->list[i]), oplist->list[i]->addr );
+            printf( "[%d]\t [%p] BIE          %ld\n", i, (void *)(oplist->list[i]), oplist->list[i]->addr );
             break;
           case F_OP_EXIT:
             printf( "[%d]\t [%p] EXIT\n", i, (void *)(oplist->list[i]) );
@@ -263,21 +272,45 @@ void ferite_opcode_dump( FeriteOpcodeList *oplist )
             printf( "[%d]\t [%p] PUSHATTR     %s(%p)\n", i, (void *)(oplist->list[i]), (char *)oplist->list[i]->opdata, oplist->list[i]->opdata );
             break;
           case F_OP_PUSHVAR:
-            printf( "[%d]\t [%p] PUSHVAR     %s(%p)\n", i, (void *)(oplist->list[i]), (char *)oplist->list[i]->opdata, oplist->list[i]->opdata );
+            printf( "[%d]\t [%p] PUSHVAR      %s(%p)\n", i, (void *)(oplist->list[i]), (char *)oplist->list[i]->opdata, oplist->list[i]->opdata );
+            break;
+          case F_OP_PUSHGLOBAL:
+            printf( "[%d]\t [%p] PUSHGLOBAL   %s:%d(%p)\n", i, (void *)(oplist->list[i]), (char *)oplist->list[i]->opdata, oplist->list[i]->addr, oplist->list[i]->opdata );
             break;
           case F_OP_PUSHINDEX:
-            printf( "[%d]\t [%p] PUSHINDEX     %ld\n", i, (void *)(oplist->list[i]), oplist->list[i]->addr );
-            break;
-          case F_OP_RGX:
-            printf( "[%d]\t [%p] RGX         %s\n", i, (void *)(oplist->list[i]), ((FeriteRegex*)(oplist->list[i]->opdata))->pattern );
+            printf( "[%d]\t [%p] PUSHINDEX    %ld\n", i, (void *)(oplist->list[i]), oplist->list[i]->addr );
             break;
           case F_OP_ERR:
-            printf( "[%d]\t [%p] ERR         %ld\n", i, (void *)(oplist->list[i]), oplist->list[i]->addr );
+            printf( "[%d]\t [%p] ERR          %ld (%s)\n", i, (void *)(oplist->list[i]), oplist->list[i]->addr, (oplist->list[i]->addr == -1 ? "reset" : "set") );
             break;
           case F_OP_CASE:
             printf( "[%d]\t [%p] CASE\n", i, (void *)(oplist->list[i]) );
             break;
-          default:
+          case F_OP_ARGS:
+            printf( "[%d]\t [%p] ARGS\n", i, (void *)(oplist->list[i]) );
+            break;
+          case F_OP_DELIVER:
+            printf( "[%d]\t [%p] DELIVER\n", i, (void *)(oplist->list[i]) );
+            break;
+          case F_OP_SET_DELIVER:
+            printf( "[%d]\t [%p] SET DELIVER (using) : %s\n", i, (void *)(oplist->list[i]), (oplist->list[i]->addr ? "Current Yield Block" : "From Object"));
+            break;
+          case F_OP_GET_DELIVER:
+            printf( "[%d]\t [%p] GET DELIVER (recipient)\n", i, (void *)(oplist->list[i]) );
+            break;
+          case F_OP_SWAP_TOP:
+            printf( "[%d]\t [%p] SWAPTOP      %d <-> %d\n", i, (void *)(oplist->list[i]), (oplist->list[i]->addr), (oplist->list[i]->addr) + 1 );
+            break;
+          case F_OP_VRST:
+            printf( "[%d]\t [%p] VRST\n", i, (void *)(oplist->list[i]) );
+            break;
+		case F_OP_RAISE:
+			printf( "[%d]\t [%p] RAISE\n", i, (void *)(oplist->list[i]) );
+			break;
+          case F_OP_NOPTOP:
+            printf( "[%d]\t [%p] NOPTOP\n", i, (void *)(oplist->list[i]) );
+            break;
+		  default:
             printf( "[%d]\t [%p] UKNOWNOP(%d)\n", i, (void *)(oplist->list[i]), oplist->list[i]->OP_TYPE );
         }
     }
@@ -311,8 +344,6 @@ FeriteOpcodeList *ferite_opcode_dup( FeriteScript *script, FeriteOpcodeList *opl
             {
                 ptr->list[i]->opdataf = fmalloc( sizeof( FeriteOpFncData ) );
                 ptr->list[i]->opdataf->argument_count = oplist->list[i]->opdataf->argument_count;
-                ptr->list[i]->opdataf->is_autoload = 0;
-                ptr->list[i]->opdataf->function = NULL;
             }
 	   else
 	     ptr->list[i]->opdataf = NULL;
@@ -340,9 +371,6 @@ FeriteOpcodeList *ferite_opcode_dup( FeriteScript *script, FeriteOpcodeList *opl
                 *value = v;
                 ptr->list[i]->opdata = value;
                 break;
-              case F_OP_RGX:
-                ptr->list[i]->opdata = ferite_regex_dup( oplist->list[i]->opdata );
-                break;
             }
         }
     }
