@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2007 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -28,21 +28,29 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#ifdef HAVE_CONFIG_HEADER
+#include "../config.h"
+#endif
+
 #include "ferite.h"
 
 /**
- * !group Stacks
- * !description Stacks are another useful data structure. Here is a stright forward implementation.
+ * @group Stacks
+ * @description Stacks are another useful data structure. Here is a stright forward implementation.
+ * @warning It must be noted that if you wish to tuck into the structure of the stack, elements are placed
+            within a 'C' array of pointers and the first element is at [1] *not* [0] as you would expect.
+            The stack_ptr variable points to the current head of the stack, for instance if stack_ptr is
+            equal to 1, then [1] has a value in it.
  */
 
 /**
- * !function ferite_create_stack
- * !declaration FeriteStack *ferite_create_stack( FeriteScript *script, int size )
- * !brief Create a stack
- * !param FeriteScript *script The script
- * !param int size The initial size of the stack
- * !return A new stack structure
- * !description It is important to note that the stacks contents go from 1 to stack->stack_ptr
+ * @function ferite_create_stack
+ * @declaration FeriteStack *ferite_create_stack( FeriteScript *script, int size )
+ * @brief Create a stack
+ * @param FeriteScript *script The script
+ * @param int size The initial size of the stack
+ * @return A new stack structure
+ * @description It is important to note that the stacks contents go from 1 to stack->stack_ptr
  */
 FeriteStack *ferite_create_stack( FeriteScript *script, int size )
 {
@@ -51,7 +59,7 @@ FeriteStack *ferite_create_stack( FeriteScript *script, int size )
 
     FE_ENTER_FUNCTION;
     if( script && script->stacks->stack_ptr )
-        ptr = ferite_stack_pop( script->stacks );
+        ptr = ferite_stack_pop( script, script->stacks );
     else
         ptr = fmalloc( sizeof( FeriteStack ) );
     ptr->size = size;
@@ -63,13 +71,13 @@ FeriteStack *ferite_create_stack( FeriteScript *script, int size )
 }
 
 /**
- * !function ferite_stack_push 
- * !declaration void ferite_stack_push( FeriteStack *stck, void *ptr )
- * !brief Push a value onto the stack
- * !param FeriteStack *stck The stack to push the value onto
- * !param void *ptr The pointer to push onto the stack
+ * @function ferite_stack_push 
+ * @declaration void ferite_stack_push( FeriteStack *stck, void *ptr )
+ * @brief Push a value onto the stack
+ * @param FeriteStack *stck The stack to push the value onto
+ * @param void *ptr The pointer to push onto the stack
  */
-void ferite_stack_push( FeriteStack *stck, void *ptr )
+void ferite_stack_push( FeriteScript *script, FeriteStack *stck, void *ptr )
 {
     int i;
 
@@ -91,31 +99,34 @@ void ferite_stack_push( FeriteStack *stck, void *ptr )
 }
 
 /**
- * !function ferite_stack_pop
- * !declaration void *ferite_stack_pop( FeriteStack *stck )
- * !brief Pop a value off the top of the stack and return it
- * !param FeriteStack *stck The stack to pop the value off
- * !return The popped off value
+ * @function ferite_stack_pop
+ * @declaration void *ferite_stack_pop( FeriteStack *stck )
+ * @brief Pop a value off the top of the stack and return it
+ * @param FeriteStack *stck The stack to pop the value off
+ * @return The popped off value
  */
-void *ferite_stack_pop( FeriteStack *stck )
+void *ferite_stack_pop( FeriteScript *script, FeriteStack *stck )
 {
     void *ptr = NULL;
 
     FE_ENTER_FUNCTION;
-    ptr = stck->stack[stck->stack_ptr];
-    stck->stack[stck->stack_ptr] = NULL;
-    stck->stack_ptr--;
-    if( stck->stack_ptr < 0 )
-      stck->stack_ptr = 0;
+    if( stck->stack_ptr >= 1 )
+    {
+        ptr = stck->stack[stck->stack_ptr];
+        stck->stack[stck->stack_ptr] = NULL;
+        stck->stack_ptr--;
+        if( stck->stack_ptr < 0 )
+            stck->stack_ptr = 0;
+    }
     FE_LEAVE_FUNCTION( ptr );
 }
 
 /**
- * !function ferite_delete_stack
- * !declaration void ferite_delete_stack( FeriteScript *script, FeriteStack *stack )
- * !brief Delete a stack and free up the memory it uses
- * !param FeriteScript *script The script
- * !param FeriteStack *stack The stack to delete
+ * @function ferite_delete_stack
+ * @declaration void ferite_delete_stack( FeriteScript *script, FeriteStack *stack )
+ * @brief Delete a stack and free up the memory it uses
+ * @param FeriteScript *script The script
+ * @param FeriteStack *stack The stack to delete
  */
 void ferite_delete_stack( FeriteScript *script, FeriteStack *stack )
 {
@@ -123,10 +134,9 @@ void ferite_delete_stack( FeriteScript *script, FeriteStack *stack )
     if( stack )
     {
         ffree( stack->stack ); /* we dont need to free the stack items, it's the job of the push
-                                * pullers to do that :)
-                                */
+                                * pullers to do that :) */
         if( script && script->stacks->stack_ptr < script->stacks->size - 1 )
-            ferite_stack_push( script->stacks, stack );
+            ferite_stack_push( script, script->stacks, stack );
         else
             ffree( stack );
     }
@@ -134,13 +144,13 @@ void ferite_delete_stack( FeriteScript *script, FeriteStack *stack )
 }
 
 /**
- * !function ferite_stack_top
- * !declaration void *ferite_stack_top( FeriteStack *stck )
- * !brief Return the top item off the stack without changing the stack's state
- * !param FeriteStack *stck The stack to get the item off
- * !return The value
+ * @function ferite_stack_top
+ * @declaration void *ferite_stack_top( FeriteStack *stck )
+ * @brief Return the top item off the stack without changing the stack's state
+ * @param FeriteStack *stck The stack to get the item off
+ * @return The value
  */
-void *ferite_stack_top( FeriteStack *stck )
+void *ferite_stack_top( FeriteScript *script, FeriteStack *stck )
 {
     void *ptr = NULL;
 
@@ -150,18 +160,18 @@ void *ferite_stack_top( FeriteStack *stck )
 }
 
 /**
- * !function ferite_duplicate_stack_contents
- * !declaration void **ferite_duplicate_stack_contents( FeriteScript *script, FeriteStack *stack, void *(*ddup)( FeriteScript*,void *data,void *data2 ), void *data2 )
- * !brief Duplicate the contents of a stack
- * !param FeriteScript *script The script
- * !param FeriteStack *stack The stack whose contents should be duplicated
- * !param void *ddup The function that gets called to copy the data
- * !return An exact copy of the stacks' contents
+ * @function ferite_duplicate_stack_contents
+ * @declaration void **ferite_duplicate_stack_contents( FeriteScript *script, FeriteStack *stack, void *(*ddup)( FeriteScript*,void *data,void *data2 ), void *data2 )
+ * @brief Duplicate the contents of a stack
+ * @param FeriteScript *script The script
+ * @param FeriteStack *stack The stack whose contents should be duplicated
+ * @param void *ddup The function that gets called to copy the data
+ * @return An exact copy of the stacks' contents
  */
 void **ferite_duplicate_stack_contents( FeriteScript *script, FeriteStack *stack, void *(*ddup)( FeriteScript*,void *data,void *data2 ), void *data2 )
 {
-    int size = stack->size + 1, i = 0;
-    void **arr = fmalloc( sizeof( void * ) * size );
+    void **arr = fmalloc( sizeof( void * ) * (stack->size + 1) );
+    int i = 0;
     
     for( i = 0; i <= stack->stack_ptr; i++ )
     {
@@ -173,6 +183,15 @@ void **ferite_duplicate_stack_contents( FeriteScript *script, FeriteStack *stack
     return arr;
 }
 
+/**
+* @function ferite_duplicate_stack
+ * @declaration FeriteStack *ferite_duplicate_stack( FeriteScript *script, FeriteStack *stack, void *(*ddup)( FeriteScript*,void *data,void *data2 ), void *data2 )
+ * @brief Duplicate a stack
+ * @param FeriteScript *script The script
+ * @param FeriteStack *stack The stack that should be duplicated
+ * @param void *ddup The function that gets called to copy the data within the stack
+ * @return An exact copy of the stacks and it's contents
+ */
 FeriteStack *ferite_duplicate_stack( FeriteScript *script, FeriteStack *stack, void *(*ddup)( FeriteScript*,void *data,void *data2 ), void *data2 )
 {
     FeriteStack *value = fmalloc( sizeof(FeriteStack) );
@@ -184,5 +203,5 @@ FeriteStack *ferite_duplicate_stack( FeriteScript *script, FeriteStack *stack, v
 }
 
 /**
- * !end
+ * @end
  */
