@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2002 Chris Ross
+ * Copyright (C) 2001-2007 Chris Ross
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,10 +27,28 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#ifdef HAVE_CONFIG_HEADER
+#include "../config.h"
+#endif
+
 #include <ferite.h>
 #include "aphex.h"
 
-FeriteThreadGroup *ferite_create_thread_group()
+/**
+ * @group Thread Group
+ * @brief This group of functions provides a set of thread safe functions for managing and tracking running threads.
+          They are mainly used to make sure that threads can validate the existence of other threads and so that the 
+          main process thread of a ferite script wont exit until all running threads have ceased execution.
+ */
+
+/**
+ * @function ferite_create_thread_group
+ * @declaration FeriteThreadGroup *ferite_create_thread_group( FeriteScript *script )
+ * @brief Create a thread group structure
+ * @param FeriteScript *script The script context
+ * @return A pointer to a FeriteThreadGroup structure on success, NULL otherwise
+ */
+FeriteThreadGroup *ferite_create_thread_group( FeriteScript *script )
 {
     FeriteThreadGroup *ptr = NULL;
 
@@ -39,10 +57,20 @@ FeriteThreadGroup *ferite_create_thread_group()
     ptr = fmalloc( sizeof( FeriteThreadGroup ) );
     ptr->lock = aphex_mutex_create();
     ptr->thread_list = ferite_create_stack( NULL, 1 );
+    ptr->owner = script;
 #endif
     FE_LEAVE_FUNCTION( ptr );
 }
 
+/**
+ * @function ferite_thread_group_destroy
+ * @declaration void ferite_thread_group_destroy( FeriteScript *script, FeriteThreadGroup *group )
+ * @brief Destroy a thread group. This function blocks.
+ * @param FeriteScript *script The script context
+ * @param FeriteThreadGroup *group The thread group to cleanup
+ * @warning This function will block the calling thread/process until all threads within the group have dettached from
+            the group/
+ */
 void ferite_thread_group_destroy( FeriteScript *script, FeriteThreadGroup *group )
 {
     FE_ENTER_FUNCTION;
@@ -59,6 +87,14 @@ void ferite_thread_group_destroy( FeriteScript *script, FeriteThreadGroup *group
     FE_LEAVE_FUNCTION( NOWT );
 }
 
+/**
+ * @function ferite_thread_group_attach
+ * @declaration void ferite_thread_group_attach( FeriteScript *script, FeriteThreadGroup *group, FeriteThread *thread )
+ * @brief Attach a given FeriteThread to a thread group.
+ * @param FeriteScript *script The script context
+ * @param FeriteThreadGroup *group The thread group to attach to
+ * @param FeriteThread *thread The thread to attach
+ */
 void ferite_thread_group_attach( FeriteScript *script, FeriteThreadGroup *group, FeriteThread *thread )
 {
     int i = 0;
@@ -69,8 +105,8 @@ void ferite_thread_group_attach( FeriteScript *script, FeriteThreadGroup *group,
     {
         aphex_mutex_lock( group->lock );
 
-    /* search the thread list for a gap */
-        for( ; i <= group->thread_list->stack_ptr; i++ )
+        /* search the thread list for a gap */
+        for( i = 1; i <= group->thread_list->stack_ptr; i++ )
         {
             if( group->thread_list->stack[i] == NULL )
             {
@@ -80,14 +116,22 @@ void ferite_thread_group_attach( FeriteScript *script, FeriteThreadGroup *group,
             }
         }
 
-    /* we didn't find a spot, put the thread at the end of the list */
-        ferite_stack_push( group->thread_list, thread );
+        /* we didn't find a spot, put the thread at the end of the list */
+        ferite_stack_push( FE_NoScript, group->thread_list, thread );
         aphex_mutex_unlock( group->lock );
     }
 #endif
     FE_LEAVE_FUNCTION(NOWT);
 }
 
+/**
+* @function ferite_thread_group_dettach
+ * @declaration void ferite_thread_group_dettach( FeriteScript *script, FeriteThreadGroup *group, FeriteThread *thread )
+ * @brief Dettach a given FeriteThread from a thread group.
+ * @param FeriteScript *script The script context
+ * @param FeriteThreadGroup *group The thread group to dettach from
+ * @param FeriteThread *thread The thread to dettach
+ */
 void ferite_thread_group_dettach( FeriteScript *script, FeriteThreadGroup *group, FeriteThread *thread )
 {
     int i = 0;
@@ -97,8 +141,8 @@ void ferite_thread_group_dettach( FeriteScript *script, FeriteThreadGroup *group
     if( group != NULL && thread != NULL )
     {
         aphex_mutex_lock( group->lock );
-    /* search the thread list, remove the thread by setting it to NULL */
-        for( ; i <= group->thread_list->stack_ptr; i++ )
+        /* search the thread list, remove the thread by setting it to NULL */
+        for( i = 1; i <= group->thread_list->stack_ptr; i++ )
         {
             if( group->thread_list->stack[i] == thread )
             {
@@ -112,6 +156,13 @@ void ferite_thread_group_dettach( FeriteScript *script, FeriteThreadGroup *group
     FE_LEAVE_FUNCTION(NOWT);
 }
 
+/**
+ * @function ferite_thread_group_wait
+ * @declaration void ferite_thread_group_wait( FeriteScript *script, FeriteThreadGroup *group )
+ * @brief Block the calling process until all threads within the group have dettached.
+ * @param FeriteScript *script The script context
+ * @param FeriteThreadGroup *group The group to wait on
+ */
 void ferite_thread_group_wait( FeriteScript *script, FeriteThreadGroup *group )
 {
     int i = 0, keep_going = FE_TRUE;
@@ -137,9 +188,11 @@ void ferite_thread_group_wait( FeriteScript *script, FeriteThreadGroup *group )
             aphex_mutex_unlock( group->lock );
             if( keep_going == FE_FALSE )
               break;
-            usleep( 100 );
+            aphex_thread_sleep( NULL, 100 );
         }
     }
 #endif
     FE_LEAVE_FUNCTION(NOWT);
 }
+
+/** @end */
