@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 Chris Ross and various contributors
+ * Copyright (C) 2000-2004 Chris Ross and various contributors
  * Copyright (C) 1999-2000 Chris Ross
  * All rights reserved.
  *
@@ -28,10 +28,67 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "config.h"
 #include <ferite.h>
 #include "aphex.h"
-#include <sys/time.h>
+#include "dot.h"
+
+#ifndef WIN32
+#ifdef HAVE_CONFIG_HEADER
+# include "config.h"
+#endif
+# include <sys/time.h>
+# include <signal.h>
+#else
+# include <windows.h>
+# include <direct.h>
+
+# ifdef NATIVE_LIBRARY_PATH
+#  undef NATIVE_LIBRARY_PATH
+# endif
+# define NATIVE_LIBRARY_DIR getLibraryPath(1)
+# ifdef XPLAT_LIBRARY_DIR
+#  undef XPLAT_LIBRARY_DIR
+# endif
+# define XPLAT_LIBRARY_DIR  getLibraryPath(2)
+# ifdef LIBRARY_DIR
+#  undef LIBRARY_DIR
+# endif
+# define LIBRARY_DIR        getLibraryPath(0)
+
+# define PACKAGE   FERITE_NAME
+# define VERSION   FERITE_VERSION
+# define PLATFORM  getWindowsPlatform()
+
+char *getWindowsPlatform()
+{
+    static char buf[1024];
+    char *b = buf;
+    sprintf( b, "Microsoft Windows %d.%d (%d)", _winmajor, _winminor, _osver );
+    return b;
+}
+
+char *getLibraryPath( int native )
+{
+    static char path[1024];
+    char buf[1024], *b = buf;
+    char *p = path;
+
+    _getcwd( b, 1024 );
+	switch( native ) {
+	case 0: /* Library dir */
+		sprintf( p, "%s", getenv("FERITE_LIBRARY_PATH") );
+		break;
+	case 1: /* Native */
+		sprintf( p, "%s", getenv("FERITE_MODULE_NATIVE_PATH") );
+		break;
+	case 2: /* xplat */
+		sprintf( p, "%s", getenv("FERITE_MODULE_XPLAT_PATH") );
+		break;
+	}
+    return p;
+}
+
+#endif
 
 struct
 {
@@ -41,17 +98,24 @@ struct
     int estring;
     int compile_only;
     int show_warnings;
+    int sleep;
+	int dot;
 }
 opt;
 
+FeriteScript *script = NULL;
+
+
 void print_version()
 {
-    printf( "%s %s [%s]\n", PACKAGE, VERSION, PLATFORM );
+    printf( "%s %s [built: %s %s] [%s]\n", PACKAGE, VERSION, __DATE__, __TIME__, PLATFORM );
+    printf( "  Base Cross Platform Library Path: %s\n", XPLAT_LIBRARY_DIR );
+    printf( "  Base Native Library Path: %s\n", NATIVE_LIBRARY_DIR ); 
 }
 
 void print_copyright()
 {
-    printf( "%s - Copyright (c) 1999-2002, Chris Ross\n", PACKAGE );
+    printf( "%s - Copyright (c) 1999-2007, Chris Ross\n", PACKAGE );
 }
 
 void print_usage()
@@ -72,9 +136,12 @@ void show_help()
     printf( " --showargs        -s        Show all the arguments passed to ferite.\n" );
     printf( " --execute         -e        Execute the script supplied on the command line.\n" );
     printf( " --include path    -Ipath    Add 'path' to ferite's search path for scripts. [effects uses's usage]\n" );
+    printf( " --native-inc path -Npath    Add 'path' to ferite's search path for native modules. [effects uses's usage]\n" );
     printf( " --preload module  -Pmodule  Get ferite to preload 'module' at compile time. [same as a 'uses' statement]\n" );
     printf( " --compile-only    -c        Only compile script - check for syntax.\n" );
-    printf( " --warnings        -w        Show warnings.\n" );
+    printf( " --hide-warnings   -w        Hide warnings.\n" );
+    printf( " --dot                       Generate a graphwiz dot file for the compiled script.\n" );
+    printf( " --sleep                     Sleep for 10 seconds once completed.\n" );
     printf( " --version                   Print the version.\n" );
     printf( " --copyright                 Print out the copyright.\n" );
     printf( "\nTo pass options to the script, add them to the end of the command line after a '--', otherwise no\n" );
@@ -88,23 +155,27 @@ int parse_args( int argc, char **argv )
     int i, gotScript = 0;
     char *buf = NULL;
 
-    opt.verbose = 0;
-    opt.showargs = 0;
-    opt.estring = 0;
-    opt.compile_only = 0;
-
+    opt.verbose = FE_FALSE;
+    opt.showargs = FE_FALSE;
+    opt.estring = FE_FALSE;
+    opt.compile_only = FE_FALSE;
+    opt.show_warnings = FE_TRUE;
+    opt.sleep = FE_FALSE;
+	opt.scriptname = NULL;
+    
     for(i = 1; i < argc; i++)
     {
         if(argv[i][0] == '-')
         {
             if( (!strcmp( argv[i], "--")) )
             {
-                return i+1;
+                argc = i+1;
+				break;
             }
             if((!strcmp(argv[i], "--verbose")) || (!strcmp(argv[i], "-v")))
-              opt.verbose = 1;
+                opt.verbose = FE_TRUE;
             if((!strcmp(argv[i], "--showargs")) || (!strcmp(argv[i], "-s")))
-              opt.showargs = 1;
+                opt.showargs = FE_TRUE;
             if((!strcmp(argv[i], "--help")) || (!strcmp(argv[i], "-h")))
             {
                 show_help();
@@ -121,11 +192,11 @@ int parse_args( int argc, char **argv )
                 exit(0);
             }
             if((!strcmp(argv[i], "--execute")) || (!strcmp(argv[i], "-e")) &&
-		    						argv[i + 1])
+               argv[i + 1])
             {
-                opt.scriptname = strdup( argv[++i] );
-                opt.estring = 1;
-                gotScript = 1;
+                opt.scriptname = aphex_strdup( argv[++i] );
+                opt.estring = FE_TRUE;
+                gotScript = FE_TRUE;
             }
             if( !strcmp(argv[i], "--include" ) )
             {
@@ -139,13 +210,33 @@ int parse_args( int argc, char **argv )
                 if( opt.verbose )
                   printf( "--> adding '%s' as a library search path\n", argv[i]+2 );
             }
+            if( !strcmp(argv[i], "--native-inc" ) )
+            {
+                ferite_set_library_native_path( argv[++i] );
+                if( opt.verbose )
+					printf( "--> adding '%s' as a native library search path\n", argv[i] );
+            }
+            if( !strncmp( argv[i], "-N", 2 ) )
+            {
+                ferite_set_library_native_path( argv[i]+2 );
+                if( opt.verbose )
+					printf( "--> adding '%s' as a native library search path\n", argv[i]+2 );
+            }
             if((!strcmp(argv[i], "--compile-only")) || (!strcmp(argv[i], "-c")))
             {
-                opt.compile_only = 1;
+                opt.compile_only = FE_TRUE;
             }
-            if((!strcmp(argv[i], "--warnings")) || (!strcmp(argv[i], "-w")))
+            if((!strcmp(argv[i], "--hide-warnings")) || (!strcmp(argv[i], "-w")))
             {
-                opt.show_warnings = 1;
+                opt.show_warnings = FE_FALSE;
+            }
+            if((!strcmp(argv[i], "--sleep")))
+            {
+                opt.sleep = FE_TRUE;
+            }
+            if((!strcmp(argv[i], "--dot")))
+            {
+                opt.dot = FE_TRUE;
             }
             if( !strcmp(argv[i], "--preload" ) )
             {
@@ -159,7 +250,6 @@ int parse_args( int argc, char **argv )
                 if( opt.verbose )
                   printf( "--> adding '%s' as a pre-loaded module\n", argv[i]+2 );
             }
-
         }
         else
         {
@@ -193,14 +283,14 @@ inline static double get_time(void)
 }
 
 int test()
-{    
+{
     FeriteScript *a = NULL, *b = NULL;
     double start = 0, end = 0, i = 0;
     double compile_diff, dup_diff;
-    
+
     start = get_time();
     for( i = 0; i < 100; i++ )
-      a = ferite_script_compile( opt.scriptname );
+        a = ferite_script_compile( opt.scriptname );
     end = get_time();
     compile_diff = end - start;
     printf( "compile each time: start %f, end %f, difference %f\n", start, end, compile_diff );
@@ -208,7 +298,7 @@ int test()
     start = get_time();
     a = ferite_script_compile( opt.scriptname );
     for( i = 0; i < 99; i++ )
-      b = ferite_duplicate_script( a );
+        b = ferite_duplicate_script( a );
     end = get_time();
     dup_diff = end - start;
     printf( "duplicate each time: start %f, end %f, difference %f\n", start, end, dup_diff );
@@ -219,11 +309,21 @@ int test()
 
 #endif
 
+void sig_ctrl( int sig )
+{
+    ferite_error( script, 0, "Ctrl+C Pressed\n" );
+}
+
 int main( int argc, char **argv )
 {
-    FeriteScript *script = NULL;
     int i = 0, retval = 0;
-    char *errmsg = NULL;
+    char *errmsg = NULL, *tmp = NULL;
+	char  dot_file[PATH_MAX];
+
+#ifndef WIN32
+    signal( SIGTERM, sig_ctrl );
+    signal( SIGINT, sig_ctrl );
+#endif
 
     if ( argc < 2 )
     {
@@ -236,96 +336,125 @@ int main( int argc, char **argv )
     else
     {
         if( opt.verbose )
-          printf( "--> initialising ferite\n");
+            printf( "--> initialising ferite\n");
 
-        /* initialise the ferite engine */
+            /* initialise the ferite engine */
         if( ferite_init( argc, argv ) )
         {
-            /* 
-             * add the system wide directory for scripts and modules eg, console.fec and sys.fec
-             * this is where the modules are looked for.
-             */
-            ferite_add_library_search_path( LIBRARY_DIR );
-            if( opt.verbose )
-              printf( "--> adding '%s' as a library search path\n", LIBRARY_DIR );
 
-            /*
-             * add the system wide directory for native modules eg. console.so and sys.so 
-             * this is where foo.lib is looked for.
-             */ 
+#ifdef AUTOLOAD_CORE
+            if( opt.verbose )
+                printf( "---> preloading core modules:\n" );
+
+            if( opt.verbose ) printf( "      :array\n" );
+            ferite_module_add_preload( "array" );
+            if( opt.verbose ) printf( "      :console\n" );
+            ferite_module_add_preload( "console" );
+            if( opt.verbose ) printf( "      :filesystem\n" );
+            ferite_module_add_preload( "filesystem" );
+            if( opt.verbose ) printf( "      :math\n" );
+            ferite_module_add_preload( "math" );
+            if( opt.verbose ) printf( "      :regexp\n" );
+            ferite_module_add_preload( "regexp" );
+            if( opt.verbose ) printf( "      :string\n" );
+            ferite_module_add_preload( "string" );
+            if( opt.verbose ) printf( "      :sys\n" );
+            ferite_module_add_preload( "sys" );
+			if( opt.verbose ) printf( "      :xml\n" );
+            ferite_module_add_preload( "xml" );	
+#endif
+			/* parse the arguments */
+            i = parse_args( argc, argv );            
+			
+			/*
+			 * add the system wide directory for scripts and modules eg, console.fec and sys.fec
+			 * this is where the modules are looked for.
+			 */
+            ferite_add_library_search_path( XPLAT_LIBRARY_DIR );
+            if( opt.verbose )
+                printf( "--> Adding '%s' as a library search path\n", LIBRARY_DIR );
+			
+			/*
+			 * add the system wide directory for native modules eg. console.so and sys.so
+			 * this is where foo.lib is looked for.
+			 */
             ferite_set_library_native_path( NATIVE_LIBRARY_DIR );
             if( opt.verbose )
-              printf( "--> setting '%s' as the native library search path\n", NATIVE_LIBRARY_DIR );
+                printf( "--> Setting '%s' as the native library search path\n", NATIVE_LIBRARY_DIR );
+			
 
-            /* parse the arguments */
-            i = parse_args( argc, argv );
-            /* setup the arguments to be passed to the scripts */
+			/* setup the arguments to be passed to the scripts */
             ferite_set_script_argv( argc - i, argv+i );
 
 #ifdef COMPARE_COMP_AND_DUP
             return test();
 #endif
-            
-            /* check to see if we should show our arguments */
+			
+			/* check to see if we should show our arguments */
             if( opt.showargs )
             {
                 printf( "Argument Count: %d %d\n", argc, i );
                 for( ; i < argc; i++ )
-                  printf( "Argument #%d: %s\n", i+1, argv[i] );
+                    printf( "Argument #%d: %s\n", i+1, argv[i] );
             }
 
             if( opt.verbose )
-              printf( "--> compiling script\n" );
+                printf( "--> Compiling script\n" );
 
             if( !opt.estring )
             {
                 char *buf = strdup( opt.scriptname );
                 char *ext = strrchr( buf, DIR_DELIM );
                 if( ext != NULL )
-                  *ext = '\0';
+                    *ext = '\0';
 
                 if( opt.verbose )
-                  printf( "  --> file '%s'\n", opt.scriptname );
+                    printf( "  --> File '%s'\n", opt.scriptname );
 
-                /* Add a library path for the directory the script lives in */
+				/* Add a library path for the directory the script lives in */
                 ferite_add_library_search_path( buf );
-                /* compile the script file */
+				/* compile the script file */
                 script = ferite_script_compile( opt.scriptname );
                 free( buf );
             }
             else
             {
                 if( opt.verbose )
-                  printf( "  --> cmd '%s'\n", opt.scriptname );
-                
+                    printf( "  --> Command '%s'\n", opt.scriptname );
+
                 ferite_set_filename( "-e" );
-                /* compile the string passed on the command line */
+                    /* compile the string passed on the command line */
                 script = ferite_compile_string( opt.scriptname );
             }
 
             /* check to see if there is a compile error */
             if( !ferite_has_compile_error( script ) )
             {
-		/* This is for checking that a script duplication works */
-		/* script = ferite_duplicate_script( script ); */
-		
-		/* If we want warnings to be shown */
-		if( ferite_has_warnings( script ) && opt.show_warnings )
-		{
-		    errmsg = ferite_get_warning_string( script );
-		    fprintf( stderr, "[ferite: compilation]\n%s", errmsg );
-		    ffree( errmsg );		    
-		}
-		ferite_reset_warnings( script );
-		
+                /* This is for checking that a script duplication works */
+                /* script = ferite_duplicate_script( script ); */
+                
+                /* If we want warnings to be shown */
+                if( ferite_has_warnings( script ) && opt.show_warnings )
+                {
+                    errmsg = ferite_get_warning_string( script );
+                    fprintf( stderr, "[ferite: compilation]\n%s", errmsg );
+                    ffree( errmsg );
+                }
+                ferite_reset_warnings( script );
+
+				if( opt.dot ) {
+					sprintf( dot_file, "%s.dot", opt.scriptname );
+					dot_write_script( script, dot_file );
+				}
+				
                 if( !opt.compile_only )
                 {
                     if( opt.verbose )
-                      printf( "--> executing script\n" );
-                    
+                        printf( "--> Executing script\n" );
+
                     /* execute script */
                     ferite_script_execute( script );
-                    
+
                     /* check to see if there is a runtime error */
                     if( ferite_has_runtime_error( script ) )
                     {
@@ -338,15 +467,15 @@ int main( int argc, char **argv )
                     else
                     {
                         retval = script->return_value;
-			
-			/* If we want warnings to be shown */
-			if( ferite_has_warnings( script ) && opt.show_warnings )
-			{
-			    errmsg = ferite_get_warning_string( script );
-			    fprintf( stderr, "[ferite: execution]\n%s", errmsg );
-			    ffree( errmsg );		    
-			}
-			ferite_reset_warnings( script );
+
+                        /* If we want warnings to be shown */
+                        if( ferite_has_warnings( script ) && opt.show_warnings )
+                        {
+                            errmsg = ferite_get_warning_string( script );
+                            fprintf( stderr, "[ferite: execution]\n%s", errmsg );
+                            ffree( errmsg );
+                        }
+                        ferite_reset_warnings( script );
                     }
                 }
                 else
@@ -356,23 +485,23 @@ int main( int argc, char **argv )
             }
             else
             {
-                /* output the compile errors */
+                    /* output the compile errors */
                 errmsg = ferite_get_error_log( script );
                 fprintf( stderr, "[ferite: compile]\n%s", errmsg );
                 ffree( errmsg );
                 if( opt.compile_only )
-                  fprintf( stderr, "FAIL: Script compilation failed.\n" );
+                    fprintf( stderr, "FAIL: Script compilation failed.\n" );
                 retval = 1;
             }
             if( opt.verbose )
-              printf( "--> destroying script object\n" );
-            
-            /* delete the script */
+                printf( "--> Destroying script object\n" );
+
+			/* delete the script */
             ferite_script_delete( script );
 
-            /* deinitialise the engien */
+			/* deinitialise the engien */
             ferite_deinit();
-            free( opt.scriptname );
+            aphex_free( opt.scriptname );
         }
         else
         {
@@ -380,9 +509,19 @@ int main( int argc, char **argv )
             retval = 4;
         }
 
+#ifndef WIN32
+        signal( SIGTERM, SIG_DFL );
+        signal( SIGINT, SIG_DFL );
+#endif
+        if( opt.sleep ) {
+            fprintf( stderr, "Sleeping...\n" );
+            sleep(2048);
+            fprintf( stderr, "Waking up...\n" );
+        }
+        
         if( opt.verbose )
-          printf( "--> Quiting Application\n" );
+            printf( "--> Quiting Application\n" );
 
-    }    
+    }
     return retval;
 }
